@@ -1,5 +1,4 @@
 from __future__ import absolute_import
-from collections import OrderedDict
 
 try:
     import itertools.izip as zip
@@ -9,11 +8,12 @@ except ImportError:
 import numpy as np
 import param
 
-from ..dimension import replace_dimensions
+from ..dimension import redim
 from .interface import Interface
 from .array import ArrayInterface
 from .dictionary import DictInterface
 from .grid import GridInterface
+from .image import ImageInterface             # noqa (API import)
 from .ndelement import NdElementInterface
 
 datatypes = ['array', 'dictionary', 'grid', 'ndelement']
@@ -46,8 +46,15 @@ try:
 except ImportError:
     pass
 
+try:
+    from .dask import DaskInterface   # noqa (Conditional API import)
+    datatypes.append('dask')
+except ImportError:
+    pass
+
 from ..dimension import Dimension
 from ..element import Element
+from ..ndmapping import OrderedDict
 from ..spaces import HoloMap, DynamicMap
 from .. import util
 
@@ -62,35 +69,68 @@ class DataConversion(object):
     def __init__(self, element):
         self._element = element
 
-    def __call__(self, new_type, kdims=None, vdims=None, mdims=None,
+    def __call__(self, new_type, kdims=None, vdims=None, groupby=None,
                  sort=False, **kwargs):
         """
-        Generic conversion method for Column based types. Supply the
-        Columns based type to convert to and optionally the
-        key dimensions (kdims), value dimensions (vdims) and HoloMap
-        key dimensions (mdims). Converted Columns can be automatically
-        sorted via the sort option and kwargs can be passed through.
+        Generic conversion method for Dataset based Element
+        types. Supply the Dataset Element type to convert to and
+        optionally the key dimensions (kdims), value dimensions
+        (vdims) and the dimensions.  to group over. Converted Columns
+        can be automatically sorted via the sort option and kwargs can
+        bepassed through.
         """
+        if 'mdims' in kwargs:
+            if groupby:
+                raise ValueError('Cannot supply both mdims and groupby')
+            else:
+                self._element.warning("'mdims' keyword has been renamed "
+                                      "to 'groupby'; the name mdims is "
+                                      "deprecated and will be removed "
+                                      "after version 1.7.")
+                groupby = kwargs.pop('mdims')
+
         if kdims is None:
-            kdims = self._element.kdims
+            kd_filter = groupby or []
+            if not isinstance(kd_filter, list):
+                kd_filter = [groupby]
+            kdims = [kd for kd in self._element.kdims if kd not in kd_filter]
         elif kdims and not isinstance(kdims, list): kdims = [kdims]
         if vdims is None:
             vdims = self._element.vdims
         if vdims and not isinstance(vdims, list): vdims = [vdims]
-        if mdims is None:
-            mdims = [d for d in self._element.kdims if d not in kdims+vdims]
 
-        selected = self._element.reindex(mdims+kdims, vdims)
-        params = {'kdims': [selected.get_dimension(kd) for kd in kdims],
-                  'vdims': [selected.get_dimension(vd) for vd in vdims],
+        # Checks Element type supports dimensionality
+        type_name = new_type.__name__
+        for dim_type, dims in (('kdims', kdims), ('vdims', vdims)):
+            min_d, max_d = new_type.params(dim_type).bounds
+            if ((min_d is not None and len(dims) < min_d) or
+                (max_d is not None and len(dims) > max_d)):
+                raise ValueError("%s %s must be between length %s and %s." %
+                                 (type_name, dim_type, min_d, max_d))
+
+        if groupby is None:
+            groupby = [d for d in self._element.kdims if d not in kdims+vdims]
+        elif groupby and not isinstance(groupby, list):
+            groupby = [groupby]
+
+        if self._element.interface.gridded:
+            dropped_kdims = [kd for kd in self._element.kdims if kd not in groupby+kdims]
+            if dropped_kdims:
+                selected = self._element.reindex(groupby+kdims, vdims)
+            else:
+                selected = self._element
+        else:
+            selected = self._element.reindex(groupby+kdims, vdims)
+        params = {'kdims': [selected.get_dimension(kd, strict=True) for kd in kdims],
+                  'vdims': [selected.get_dimension(vd, strict=True) for vd in vdims],
                   'label': selected.label}
         if selected.group != selected.params()['group'].default:
             params['group'] = selected.group
         params.update(kwargs)
-        if len(kdims) == selected.ndims:
+        if len(kdims) == selected.ndims or not groupby:
             element = new_type(selected, **params)
             return element.sort() if sort else element
-        group = selected.groupby(mdims, container_type=HoloMap,
+        group = selected.groupby(groupby, container_type=HoloMap,
                                  group_type=new_type, **params)
         if sort:
             return group.map(lambda x: x.sort(), [new_type])
@@ -119,24 +159,36 @@ class Dataset(Element):
 
     # In the 1D case the interfaces should not automatically add x-values
     # to supplied data
-    _1d = False
+    _auto_indexable_1d = True
 
     # Define a class used to transform Datasets into other Element types
     _conversion_interface = DataConversion
 
+    _vdim_reductions = {}
+    _kdim_reductions = {}
+
     def __init__(self, data, **kwargs):
         if isinstance(data, Element):
             pvals = util.get_param_values(data)
             kwargs.update([(l, pvals[l]) for l in ['group', 'label']
                            if l in pvals and l not in kwargs])
-        initialized = Interface.initialize(type(self), data,
-                                           kwargs.get('kdims'),
-                                           kwargs.get('vdims'),
+
+        kdims, vdims = None, None
+        if 'kdims' in kwargs:
+            kdims = [kd if isinstance(kd, Dimension) else Dimension(kd)
+                     for kd in kwargs['kdims']]
+        if 'vdims' in kwargs:
+            vdims = [kd if isinstance(kd, Dimension) else Dimension(kd)
+                     for kd in kwargs['vdims']]
+
+        initialized = Interface.initialize(type(self), data, kdims, vdims,
                                            datatype=kwargs.get('datatype'))
         (data, self.interface, dims, extra_kws) = initialized
-        super(Dataset, self).__init__(data, **dict(extra_kws, **dict(kwargs, **dims)))
+        super(Dataset, self).__init__(data, **dict(kwargs, **dict(dims, **extra_kws)))
         self.interface.validate(self)
 
+        self.redim = redim(self, mode='dataset')
+
 
     def __setstate__(self, state):
         """
@@ -156,19 +208,32 @@ class Dataset(Element):
 
         super(Dataset, self).__setstate__(state)
 
-    def closest(self, coords):
+
+    def closest(self, coords=[], **kwargs):
         """
-        Given single or multiple samples along the first key dimension
-        will return the closest actual sample coordinates.
+        Given a single coordinate or multiple coordinates as
+        a tuple or list of tuples or keyword arguments matching
+        the dimension closest will find the closest actual x/y
+        coordinates. Different Element types should implement this
+        appropriately depending on the space they represent, if the
+        Element does not support snapping raise NotImplementedError.
         """
         if self.ndims > 1:
-            NotImplementedError("Closest method currently only "
-                                "implemented for 1D Elements")
+            raise NotImplementedError("Closest method currently only "
+                                      "implemented for 1D Elements")
+
+        if kwargs:
+            if len(kwargs) > 1:
+                raise NotImplementedError("Closest method currently only "
+                                          "supports 1D indexes")
+            samples = list(kwargs.values())[0]
+            coords = samples if isinstance(samples, list) else [samples]
 
-        if not isinstance(coords, list): coords = [coords]
         xs = self.dimension_values(0)
+        if xs.dtype.kind in 'SO':
+            raise NotImplementedError("Closest only supported for numeric types")
         idxs = [np.argmin(np.abs(xs-coord)) for coord in coords]
-        return [xs[idx] for idx in idxs] if len(coords) > 1 else xs[idxs[0]]
+        return [xs[idx] for idx in idxs]
 
 
     def sort(self, by=[]):
@@ -176,6 +241,8 @@ class Dataset(Element):
         Sorts the data by the values along the supplied dimensions.
         """
         if not by: by = self.kdims
+        if not isinstance(by, list): by = [by]
+
         sorted_columns = self.interface.sort(self, by)
         return self.clone(sorted_columns)
 
@@ -187,21 +254,27 @@ class Dataset(Element):
         object.
         """
         dim = self.get_dimension(dim)
-        if dim.range != (None, None):
+        if dim is None:
+            return (None, None)
+        elif None not in dim.range:
             return dim.range
-        elif dim in self.dimensions():
+        elif dim in self.dimensions() and data_range:
             if len(self):
                 drange = self.interface.range(self, dim)
             else:
                 drange = (np.NaN, np.NaN)
-        if data_range:
             soft_range = [r for r in dim.soft_range if r is not None]
             if soft_range:
-                return util.max_range([drange, soft_range])
-            else:
-                return drange
+                drange = util.max_range([drange, soft_range])
         else:
-            return dim.soft_range
+            drange = dim.soft_range
+        if dim.range[0] is not None:
+            return (dim.range[0], drange[1])
+        elif dim.range[1] is not None:
+            return (drange[0], dim.range[1])
+        else:
+            return drange
+
 
 
     def add_dimension(self, dimension, dim_pos, dim_val, vdim=False, **kwargs):
@@ -211,7 +284,7 @@ class Dataset(Element):
         dimensions and a key value scalar or sequence of the same length
         as the existing keys.
         """
-        if isinstance(dimension, str):
+        if isinstance(dimension, (util.basestring, tuple)):
             dimension = Dimension(dimension)
 
         if dimension.name in self.kdims:
@@ -241,10 +314,14 @@ class Dataset(Element):
         supplied, which will ensure the selection is only applied if the
         specs match the selected object.
         """
-        if selection_specs and not any(self.matches(sp) for sp in selection_specs):
+        selection = {dim: sel for dim, sel in selection.items()
+                     if dim in self.dimensions()+['selection_mask']}
+        if (selection_specs and not any(self.matches(sp) for sp in selection_specs)
+            or not selection):
             return self
 
         data = self.interface.select(self, **selection)
+
         if np.isscalar(data):
             return data
         else:
@@ -259,15 +336,18 @@ class Dataset(Element):
         if kdims is None:
             key_dims = [d for d in self.kdims if not vdims or d not in vdims]
         else:
-            key_dims = [self.get_dimension(k) for k in kdims]
+            key_dims = [self.get_dimension(k, strict=True) for k in kdims]
 
+        new_type = None
         if vdims is None:
             val_dims = [d for d in self.vdims if not kdims or d not in kdims]
         else:
-            val_dims = [self.get_dimension(v) for v in vdims]
+            val_dims = [self.get_dimension(v, strict=True) for v in vdims]
+            new_type = self._vdim_reductions.get(len(val_dims), type(self))
 
         data = self.interface.reindex(self, key_dims, val_dims)
-        return self.clone(data, kdims=key_dims, vdims=val_dims)
+        return self.clone(data, kdims=key_dims, vdims=val_dims,
+                          new_type=new_type)
 
 
     def __getitem__(self, slices):
@@ -290,7 +370,7 @@ class Dataset(Element):
         if isinstance(slices, np.ndarray) and slices.dtype.kind == 'b':
             if not len(slices) == len(self):
                 raise IndexError("Boolean index must match length of sliced object")
-            return self.clone(self.interface.select(self, selection_mask=slices))
+            return self.clone(self.select(selection_mask=slices))
         elif slices in [(), Ellipsis]:
             return self
         if not isinstance(slices, tuple): slices = (slices,)
@@ -302,25 +382,70 @@ class Dataset(Element):
             value_select = slices[self.ndims]
         elif len(slices) == self.ndims+1 and isinstance(slices[self.ndims],
                                                         (Dimension,str)):
-            raise Exception("%r is not an available value dimension'" % slices[self.ndims])
+            raise Exception("%r is not an available value dimension" % slices[self.ndims])
         else:
             selection = dict(zip(self.dimensions(label=True), slices))
         data = self.select(**selection)
         if value_select:
-            if len(data) == 1:
+            if data.shape[0] == 1:
                 return data[value_select][0]
             else:
                 return data.reindex(vdims=[value_select])
         return data
 
 
-    def sample(self, samples=[]):
+    def sample(self, samples=[], closest=True, **kwargs):
         """
         Allows sampling of Dataset as an iterator of coordinates
         matching the key dimensions, returning a new object containing
-        just the selected samples.
+        just the selected samples. Alternatively may supply kwargs
+        to sample a coordinate on an object. By default it will attempt
+        to snap to the nearest coordinate if the Element supports it,
+        snapping may be disabled with the closest argument.
         """
-        return self.clone(self.interface.sample(self, samples))
+        if kwargs and samples:
+            raise Exception('Supply explicit list of samples or kwargs, not both.')
+        elif kwargs:
+            sample = [slice(None) for _ in range(self.ndims)]
+            for dim, val in kwargs.items():
+                sample[self.get_dimension_index(dim)] = val
+            samples = [tuple(sample)]
+
+        # Note: Special handling sampling of gridded 2D data as Curve
+        # may be replaced wih more general handling
+        # see https://github.com/ioam/holoviews/issues/1173
+        from ...element import Table, Curve
+        if len(samples) == 1:
+            sel = {kd.name: s for kd, s in zip(self.kdims, samples[0])}
+            dims = [kd for kd, v in sel.items() if not np.isscalar(v)]
+            selection = self.select(**sel)
+
+            # If a 1D cross-section of 2D space return Curve
+            if self.interface.gridded and self.ndims == 2 and len(dims) == 1:
+                new_type = Curve
+                kdims = [self.get_dimension(kd) for kd in dims]
+            else:
+                new_type = Table
+                kdims = self.kdims
+
+            if np.isscalar(selection):
+                selection = [samples[0]+(selection,)]
+            else:
+                selection = tuple(selection.columns(kdims+self.vdims).values())
+
+            return self.clone(selection, kdims=kdims, new_type=new_type)
+
+        lens = set(len(util.wrap_tuple(s)) for s in samples)
+        if len(lens) > 1:
+            raise IndexError('Sample coordinates must all be of the same length.')
+
+        if closest:
+            try:
+                samples = self.closest(samples)
+            except NotImplementedError:
+                pass
+        samples = [util.wrap_tuple(s) for s in samples]
+        return self.clone(self.interface.sample(self, samples), new_type=Table)
 
 
     def reduce(self, dimensions=[], function=None, spreadfn=None, **reduce_map):
@@ -346,28 +471,38 @@ class Dataset(Element):
             raise ValueError("The aggregate method requires a function to be specified")
         if dimensions is None: dimensions = self.kdims
         elif not isinstance(dimensions, list): dimensions = [dimensions]
-        aggregated = self.interface.aggregate(self, dimensions, function, **kwargs)
+        kdims = [self.get_dimension(d, strict=True) for d in dimensions]
+        aggregated = self.interface.aggregate(self, kdims, function, **kwargs)
         aggregated = self.interface.unpack_scalar(self, aggregated)
 
-        kdims = [self.get_dimension(d) for d in dimensions]
+        ndims = len(dimensions)
+        min_d, max_d = self.params('kdims').bounds
+        generic_type = (min_d is not None and ndims < min_d) or (max_d is not None and ndims > max_d)
+
         vdims = self.vdims
         if spreadfn:
             error = self.interface.aggregate(self, dimensions, spreadfn)
             spread_name = spreadfn.__name__
             ndims = len(vdims)
-            error = self.clone(error, kdims=kdims)
-            combined = self.clone(aggregated, kdims=kdims)
+            error = self.clone(error, kdims=kdims, new_type=Dataset)
+            combined = self.clone(aggregated, kdims=kdims, new_type=Dataset)
             for i, d in enumerate(vdims):
                 dim = d('_'.join([d.name, spread_name]))
                 dvals = error.dimension_values(d, False, False)
                 combined = combined.add_dimension(dim, ndims+i, dvals, True)
-            return combined
+            return combined.clone(new_type=Dataset if generic_type else type(self))
 
         if np.isscalar(aggregated):
             return aggregated
         else:
-            return self.clone(aggregated, kdims=kdims, vdims=vdims)
-
+            try:
+                # Should be checking the dimensions declared on the element are compatible
+                return self.clone(aggregated, kdims=kdims, vdims=vdims)
+            except:
+                datatype = self.params('datatype').default
+                return self.clone(aggregated, kdims=kdims, vdims=vdims,
+                                  new_type=Dataset if generic_type else None,
+                                  datatype=datatype)
 
 
     def groupby(self, dimensions=[], container_type=HoloMap, group_type=None,
@@ -394,15 +529,20 @@ class Dataset(Element):
 
         if dynamic:
             group_dims = [d.name for d in self.kdims if d not in dimensions]
-            group_kwargs = dict(util.get_param_values(self), **kwargs)
-            group_kwargs['kdims'] = [self.get_dimension(d) for d in group_dims]
+            kdims = [self.get_dimension(d) for d in group_dims]
+            group_kwargs = dict(util.get_param_values(self), kdims=kdims)
+            group_kwargs.update(kwargs)
+            drop_dim = len(kdims) != len(group_kwargs['kdims'])
             def load_subset(*args):
                 constraint = dict(zip(dim_names, args))
                 group = self.select(**constraint)
                 if np.isscalar(group):
                     return group_type(([group],), group=self.group,
                                       label=self.label, vdims=self.vdims)
-                return group_type(group.reindex(group_dims), **group_kwargs)
+                data = group.reindex(group_dims)
+                if drop_dim and self.interface.gridded:
+                    data = data.columns()
+                return group_type(data, **group_kwargs)
             dynamic_dims = [d(values=list(self.interface.values(self, d.name, False)))
                             for d in dimensions]
             return DynamicMap(load_subset, kdims=dynamic_dims)
@@ -416,6 +556,10 @@ class Dataset(Element):
         """
         return self.interface.length(self)
 
+    def __nonzero__(self):
+        return self.interface.nonzero(self)
+
+    __bool__ = __nonzero__
 
     @property
     def shape(self):
@@ -423,35 +567,12 @@ class Dataset(Element):
         return self.interface.shape(self)
 
 
-    def redim(self, specs=None, **dimensions):
-        """
-        Replace dimensions on the dataset and allows renaming
-        dimensions in the dataset. Dimension mapping should map
-        between the old dimension name and a dictionary of the new
-        attributes, a completely new dimension or a new string name.
-        """
-        if specs is not None:
-            if not isinstance(specs, list):
-                specs = [specs]
-            if not any(self.matches(spec) for spec in specs):
-                return self
-
-        kdims = replace_dimensions(self.kdims, dimensions)
-        vdims = replace_dimensions(self.vdims, dimensions)
-        zipped_dims = zip(self.kdims+self.vdims, kdims+vdims)
-        renames = {pk.name: nk for pk, nk in zipped_dims if pk != nk}
-        data = self.data
-        if renames:
-            data = self.interface.redim(self, renames)
-        return self.clone(data, kdims=kdims, vdims=vdims)
-
-
     def dimension_values(self, dim, expanded=True, flat=True):
         """
         Returns the values along a particular dimension. If unique
         values are requested will return only unique values.
         """
-        dim = self.get_dimension(dim, strict=True).name
+        dim = self.get_dimension(dim, strict=True)
         return self.interface.values(self, dim, expanded, flat)
 
 
@@ -469,17 +590,21 @@ class Dataset(Element):
 
     def dframe(self, dimensions=None):
         """
-        Returns the data in the form of a DataFrame.
+        Returns the data in the form of a DataFrame. Supplying a list
+        of dimensions filters the dataframe. If the data is already
+        a DataFrame a copy is returned.
         """
         if dimensions:
-            dimensions = [self.get_dimension(d).name for d in dimensions]
+            dimensions = [self.get_dimension(d, strict=True).name for d in dimensions]
         return self.interface.dframe(self, dimensions)
 
 
     def columns(self, dimensions=None):
-        if dimensions is None: dimensions = self.dimensions()
-        dimensions = [self.get_dimension(d) for d in dimensions]
-        return {d.name: self.dimension_values(d) for d in dimensions}
+        if dimensions is None:
+            dimensions = self.dimensions()
+        else:
+            dimensions = [self.get_dimension(d, strict=True) for d in dimensions]
+        return OrderedDict([(d.name, self.dimension_values(d)) for d in dimensions])
 
 
     @property
