@@ -45,6 +45,8 @@ class ArrayInterface(Interface):
                 data = np.column_stack(data)
             else:
                 raise ValueError('ArrayInterface expects data to be of uniform shape.')
+        elif isinstance(data, list) and data == []:
+            data = np.empty((0,len(dimensions)))
         elif not isinstance(data, np.ndarray):
             data = np.array([], ndmin=2).T if data is None else list(data)
             try:
@@ -55,10 +57,10 @@ class ArrayInterface(Interface):
         if data is None or data.ndim > 2 or data.dtype.kind in ['S', 'U', 'O']:
             raise ValueError("ArrayInterface interface could not handle input type.")
         elif data.ndim == 1:
-            if eltype._1d:
-                data = np.atleast_2d(data).T
-            else:
+            if eltype._auto_indexable_1d:
                 data = np.column_stack([np.arange(len(data)), data])
+            else:
+                data = np.atleast_2d(data).T
 
         if kdims is None:
             kdims = eltype.kdims
@@ -131,7 +133,7 @@ class ArrayInterface(Interface):
         data = dataset.data
 
         # Get dimension objects, labels, indexes and data
-        dimensions = [dataset.get_dimension(d) for d in dimensions]
+        dimensions = [dataset.get_dimension(d, strict=True) for d in dimensions]
         dim_idxs = [dataset.get_dimension_index(d) for d in dimensions]
         ndims = len(dimensions)
         kdims = [kdim for kdim in dataset.kdims
@@ -183,7 +185,7 @@ class ArrayInterface(Interface):
             selection_mask = cls.select_mask(dataset, selection)
         indexed = cls.indexed(dataset, selection)
         data = np.atleast_2d(dataset.data[selection_mask, :])
-        if len(data) == 1 and indexed:
+        if len(data) == 1 and indexed and len(dataset.vdims) == 1:
             data = data[0, dataset.ndims]
         return data
 
