@@ -14,11 +14,10 @@ import param
 from ..core.util import (basestring, sanitize_identifier,
                          group_sanitizer, label_sanitizer, max_range,
                          find_range, dimension_sanitizer, OrderedDict,
-                         safe_unicode, unicode, dt64_to_dt)
+                         bytes_to_unicode, unicode, dt64_to_dt, unique_array)
 from .options import Store, StoreOptions
 from .pprint import PrettyPrinter
 
-
 # Alias parameter support for pickle loading
 
 ALIASES = {'key_dimensions': 'kdims', 'value_dimensions': 'vdims',
@@ -41,33 +40,138 @@ def param_aliases(d):
     return d
 
 
-def replace_dimensions(dimensions, overrides):
+class redim(object):
     """
-    Replaces dimensions in a list with a dictionary of overrides.
-    Overrides should be indexed by the dimension name with values that
-    is either a Dimension object, a string name or a dictionary
-    specifying the dimension parameters to override.
+    Utility that supports re-dimensioning any HoloViews object via the
+    redim method.
     """
-    replaced = []
-    for d in dimensions:
-        if d.name in overrides:
-            override = overrides[d.name]
-        else:
-            override = None
 
-        if override is None:
-            replaced.append(d)
-        elif isinstance(override, basestring):
-            replaced.append(d(override))
-        elif isinstance(override, Dimension):
-            replaced.append(override)
-        elif isinstance(override, dict):
-            replaced.append(d(**override))
-        else:
-            raise ValueError('Dimension can only be overridden '
-                             'with another dimension or a dictionary '
-                             'of attributes')
-    return replaced
+    def __init__(self, parent, mode=None):
+        self.parent = parent
+        # Can be 'dataset', 'dynamic' or None
+        self.mode = mode
+
+    def __str__(self):
+        return "<holoviews.core.dimension.redim method>"
+
+    @classmethod
+    def replace_dimensions(cls, dimensions, overrides):
+        """
+        Replaces dimensions in a list with a dictionary of overrides.
+        Overrides should be indexed by the dimension name with values that
+        is either a Dimension object, a string name or a dictionary
+        specifying the dimension parameters to override.
+        """
+        replaced = []
+        for d in dimensions:
+            if d.name in overrides:
+                override = overrides[d.name]
+            elif d.label in overrides:
+                override = overrides[d.label]
+            else:
+                override = None
+
+            if override is None:
+                replaced.append(d)
+            elif isinstance(override, (basestring, tuple)):
+                replaced.append(d(override))
+            elif isinstance(override, Dimension):
+                replaced.append(override)
+            elif isinstance(override, dict):
+                replaced.append(d.clone(override.get('name',None),
+                                        **{k:v for k,v in override.items() if k != 'name'}))
+            else:
+                raise ValueError('Dimension can only be overridden '
+                                 'with another dimension or a dictionary '
+                                 'of attributes')
+        return replaced
+
+
+    def _filter_cache(self, dmap, kdims):
+        """
+        Returns a filtered version of the DynamicMap cache leaving only
+        keys consistently with the newly specified values
+        """
+        filtered = []
+        for key, value in dmap.data.items():
+            if not any(kd.values and v not in kd.values for kd, v in zip(kdims, key)):
+                filtered.append((key, value))
+        return filtered
+
+
+    def __call__(self, specs=None, **dimensions):
+        """
+        Replace dimensions on the dataset and allows renaming
+        dimensions in the dataset. Dimension mapping should map
+        between the old dimension name and a dictionary of the new
+        attributes, a completely new dimension or a new string name.
+        """
+        parent = self.parent
+        redimmed = parent
+        if parent._deep_indexable and self.mode != 'dataset':
+            deep_mapped = [(k, v.redim(specs, **dimensions))
+                           for k, v in parent.items()]
+            redimmed = parent.clone(deep_mapped)
+
+        if specs is not None:
+            if not isinstance(specs, list):
+                specs = [specs]
+            matches = any(parent.matches(spec) for spec in specs)
+            if self.mode != 'dynamic' and not matches:
+                return redimmed
+
+
+        kdims = self.replace_dimensions(parent.kdims, dimensions)
+        vdims = self.replace_dimensions(parent.vdims, dimensions)
+        zipped_dims = zip(parent.kdims+parent.vdims, kdims+vdims)
+        renames = {pk.name: nk for pk, nk in zipped_dims if pk != nk}
+
+        if self.mode == 'dataset':
+            data = parent.data
+            if renames:
+                data = parent.interface.redim(parent, renames)
+            return parent.clone(data, kdims=kdims, vdims=vdims)
+
+        if self.mode != 'dynamic':
+            return redimmed.clone(kdims=kdims, vdims=vdims)
+
+        redimmed = redimmed.clone(kdims=kdims, vdims=vdims,
+                                  data= self._filter_cache(redimmed, kdims))
+        from ..util import Dynamic
+        def dynamic_redim(obj, **dynkwargs):
+            return obj.redim(specs, **dimensions)
+        return Dynamic(redimmed, shared_data=True, streams=parent.streams,
+                       operation=dynamic_redim)
+
+
+    def _redim(self, name, specs, **dims):
+        dimensions = {k:{name:v} for k,v in dims.items()}
+        return self(specs, **dimensions)
+
+    def cyclic(self, specs=None, **values):
+        return self._redim('cyclic', specs, **values)
+
+    def value_format(self, specs=None, **values):
+        return self._redim('value_format', specs, **values)
+
+    def range(self, specs=None, **values):
+        return self._redim('range', specs, **values)
+
+    def soft_range(self, specs=None, **values):
+        return self._redim('soft_range', specs, **values)
+
+    def type(self, specs=None, **values):
+        return self._redim('type', specs, **values)
+
+    def step(self, specs=None, **values):
+        return self._redim('step', specs, **values)
+
+    def unit(self, specs=None, **values):
+        return self._redim('unit', specs, **values)
+
+    def values(self, specs=None, **ranges):
+        return self._redim('values', specs, **ranges)
+
 
 
 class Dimension(param.Parameterized):
@@ -77,16 +181,54 @@ class Dimension(param.Parameterized):
 
     For instance, a Dimension may specify that a set of numeric values
     actually correspond to 'Height' (dimension name), in units of
-    meters, and that allowed values must be floats greater than zero.
+    meters, with a descriptive label 'Height of adult males'.
 
-    In addition, Dimensions can be declared as cyclic, support
-    categorical data using a finite set of allowed, ordered values and
-    support a custom, pretty-printed representation.
+    All dimensions object have a name that identifies them and a label
+    containing a suitable description. If the label is not explicitly
+    specified it matches the name.
+
+    These two parameters define the core identity of the dimension
+    object and must match if two dimension objects are to be considered
+    equivalent. All other parameters are considered optional metadata
+    and are not used when testing for equality.
+
+    Unlike all the other parameters, these core parameters can be used
+    to construct a Dimension object from a tuple. This format is
+    sufficient to define an identical Dimension:
+
+    Dimension('a', label='Dimension A') == Dimension(('a', 'Dimension A'))
+
+    Everything else about a dimension is considered to reflect
+    non-semantic preferences. Examples include the default value (which
+    may be used in a visualization to set an initial slider position),
+    how the value is to rendered as text (which may be used to specify
+    the printed floating point precision) or a suitable range of values
+    to consider for a particular analysis.
+
+    Units
+    -----
+
+    Full unit support with automated conversions are on the HoloViews
+    roadmap. Once rich unit objects are supported, the unit (or more
+    specifically the type of unit) will be part of the core dimension
+    specification used to establish equality.
+
+    Until this feature is implemented, there are two auxillary
+    parameters that hold some partial information about the unit: the
+    name of the unit and whether or not it is cyclic. The name of the
+    unit is used as part of the pretty-printed representation and
+    knowing whether it is cyclic is important for certain operations.
     """
 
     name = param.String(doc="""
-        Optional name associated with the Dimension. For instance,
-        'height' or 'weight'.""")
+       Short name associated with the Dimension, such as 'height' or
+       'weight'. Valid Python identifiers make good names, because they
+       can be used conveniently as a keyword in many contexts.""")
+
+    label = param.String(default=None, doc="""
+        Unrestricted label used to describe the dimension. A label
+        should succinctly describe the dimension and may contain any
+        characters, including Unicode and LaTeX expression.""")
 
     cyclic = param.Boolean(default=False, doc="""
         Whether the range of this feature is cyclic such that the
@@ -109,15 +251,20 @@ class Dimension(param.Parameterized):
         may be an inbuilt constructor (such as int, str, float) or a
         custom class object.""")
 
+    step = param.Number(default=None, doc="""
+        Optional floating point step specifying how frequently the
+        underlying space should be sampled. May be used to define a
+        discrete sampling of over the range.""")
+
     unit = param.String(default=None, allow_None=True, doc="""
         Optional unit string associated with the Dimension. For
         instance, the string 'm' may be used represent units of meters
         and 's' to represent units of seconds.""")
 
-    values = param.ClassSelector(class_=(str, list), default=[], doc="""
-        Optional set of allowed values for the dimension that can also
-        be used to retain a categorical ordering. Setting values to
-        'initial' indicates that the values will be added during construction.""")
+    values = param.List(default=[], doc="""
+        Optional specification of the allowed value set for the
+        dimension that may also be used to retain a categorical
+        ordering.""")
 
     # Defines default formatting by type
     type_formatters = {}
@@ -125,47 +272,136 @@ class Dimension(param.Parameterized):
     presets = {} # A dictionary-like mapping name, (name,) or
                  # (name, unit) to a preset Dimension object
 
-    def __init__(self, name, **params):
+    def __init__(self, spec, **params):
         """
         Initializes the Dimension object with the given name.
         """
-        if isinstance(name, Dimension):
-            existing_params = dict(name.get_param_values())
-        elif (name, params.get('unit', None)) in self.presets.keys():
-            preset = self.presets[(str(name), str(params['unit']))]
+        if 'name' in params:
+            raise KeyError('Dimension name must only be passed as the positional argument')
+
+        if isinstance(spec, Dimension):
+            existing_params = dict(spec.get_param_values())
+        elif (spec, params.get('unit', None)) in self.presets.keys():
+            preset = self.presets[(str(spec), str(params['unit']))]
             existing_params = dict(preset.get_param_values())
-        elif name in self.presets.keys():
-            existing_params = dict(self.presets[str(name)].get_param_values())
-        elif (name,) in self.presets.keys():
-            existing_params = dict(self.presets[(str(name),)].get_param_values())
+        elif spec in self.presets:
+            existing_params = dict(self.presets[spec].get_param_values())
+        elif (spec,) in self.presets:
+            existing_params = dict(self.presets[(spec,)].get_param_values())
         else:
-            existing_params = {'name': name}
+            existing_params = {}
 
         all_params = dict(existing_params, **params)
-        if isinstance(all_params['name'], tuple):
-            alias, long_name = all_params['name']
-            dimension_sanitizer.add_aliases(**{alias:long_name})
-            all_params['name'] = long_name
+        if isinstance(spec, tuple):
+            name, label = spec
+            all_params['name'] = name
+            all_params['label'] = label
+            if 'label' in params and (label != params['label']):
+                self.warning('Using label as supplied by keyword ({!r}), ignoring '
+                             'tuple value {!r}'.format(params['label'], label))
+                all_params['label'] = params['label']
+        elif isinstance(spec, basestring):
+            all_params['name'] = spec
+            all_params['label'] = params.get('label', spec)
 
+        if all_params['name'] == '':
+            raise ValueError('Dimension name cannot be the empty string')
+        if all_params['label'] in ['', None]:
+            raise ValueError('Dimension label cannot be None or the empty string')
+
+        values = params.get('values', [])
+        if isinstance(values, basestring) and values == 'initial':
+            self.warning("The 'initial' string for dimension values is no longer supported.")
+            values = []
+
+        all_params['values'] = sorted(list(unique_array(values)))
         super(Dimension, self).__init__(**all_params)
 
 
-    def __call__(self, name=None, **overrides):
+    @property
+    def spec(self):
+        "Returns the corresponding tuple specification"
+        return (self.name, self.label)
+
+
+    def __call__(self, spec=None, **overrides):
+        "Aliased to clone method. To be deprecated in 2.0"
+        return self.clone(spec=spec, **overrides)
+
+
+    def clone(self, spec=None, **overrides):
         """
         Derive a new Dimension that inherits existing parameters
         except for the supplied, explicit overrides
         """
         settings = dict(self.get_param_values(onlychanged=True), **overrides)
-        if name is not None: settings['name'] = name
-        return self.__class__(**settings)
 
+        if spec is None:
+            spec = (self.name, overrides.get('label', self.label))
+        if 'label' in overrides and isinstance(spec, basestring) :
+            spec = (spec, overrides['label'])
+        elif 'label' in overrides and isinstance(spec, tuple) :
+            self.warning('Using label as supplied by keyword ({!r}), ignoring '
+                             'tuple value {!r}'.format(overrides['label'], spec[1]))
+            spec = (spec[0],  overrides['label'])
+
+        return self.__class__(spec, **{k:v for k,v in settings.items()
+                                       if k not in ['name', 'label']})
+
+    def __hash__(self):
+        """
+        The hash allows Dimension objects to be used as dictionary keys in Python 3.
+        """
+        return hash(self.spec)
+
+    def __setstate__(self, d):
+        """
+        Compatibility for pickles before alias attribute was introduced.
+        """
+        super(Dimension, self).__setstate__(d)
+        self.label = self.name
+
+    def __eq__(self, other):
+        "Implements equals operator including sanitized comparison."
+
+        if isinstance(other, Dimension):
+            return self.spec == other.spec
+
+        # For comparison to strings. Name may be sanitized.
+        return other in [self.name, self.label,  dimension_sanitizer(self.name)]
+
+    def __ne__(self, other):
+        "Implements not equal operator including sanitized comparison."
+        return not self.__eq__(other)
+
+    def __lt__(self, other):
+        "Dimensions are sorted alphanumerically by name"
+        return self.name < other.name if isinstance(other, Dimension) else self.name < other
+
+    def __str__(self):
+        return self.name
+
+    def __repr__(self):
+        return self.pprint()
 
     @property
     def pprint_label(self):
         "The pretty-printed label string for the Dimension"
         unit = ('' if self.unit is None
                 else type(self.unit)(self.unit_format).format(unit=self.unit))
-        return safe_unicode(self.name) + safe_unicode(unit)
+        return bytes_to_unicode(self.label) + bytes_to_unicode(unit)
+
+    def pprint(self):
+        changed = dict(self.get_param_values(onlychanged=True))
+        if len(set([changed.get(k, k) for k in ['name','label']])) == 1:
+            return 'Dimension({spec})'.format(spec=repr(self.name))
+
+        ordering = sorted( sorted(changed.keys()),
+                           key=lambda k: (- float('inf')
+                                          if self.params(k).precedence is None
+                                          else self.params(k).precedence))
+        kws = ", ".join('%s=%r' % (k, changed[k]) for k in ordering if k != 'name')
+        return 'Dimension({spec}, {kws})'.format(spec=repr(self.name), kws=kws)
 
 
     def pprint_value(self, value):
@@ -187,12 +423,7 @@ class Dimension(param.Parameterized):
                     return formatter.format(value)
                 else:
                     return formatter % value
-        return value
-
-
-    def __repr__(self):
-        return self.pprint()
-
+        return unicode(bytes_to_unicode(value))
 
     def pprint_value_string(self, value):
         """
@@ -200,38 +431,9 @@ class Dimension(param.Parameterized):
         title_format variable, including the unit string (if
         set). Numeric types are printed to the stated rounding level.
         """
-        unit = '' if self.unit is None else ' ' + self.unit
+        unit = '' if self.unit is None else ' ' + bytes_to_unicode(self.unit)
         value = self.pprint_value(value)
-        return title_format.format(name=self.name, val=value, unit=unit)
-
-
-    def __hash__(self):
-        """
-        The hash allows two Dimension objects to be compared; if the
-        hashes are equal, all the parameters of the Dimensions are
-        also equal.
-        """
-        return sum([hash(value) for _, value in self.get_param_values()
-                    if not isinstance(value, list)])
-
-
-    def __str__(self):
-        return self.pprint_label
-
-
-    def __eq__(self, other):
-        "Implements equals operator including sanitized comparison."
-        dim_matches = [self.name, dimension_sanitizer(self.name)]
-        return other.name in dim_matches if isinstance(other, Dimension) else other in dim_matches
-
-    def __ne__(self, other):
-        "Implements not equal operator including sanitized comparison."
-        return not self.__eq__(other)
-
-    def __lt__(self, other):
-        "Dimensions are sorted alphanumerically by name"
-        return self.name < other.name if isinstance(other, Dimension) else self.name < other
-
+        return title_format.format(name=bytes_to_unicode(self.label), val=value, unit=unit)
 
 
 class LabelledData(param.Parameterized):
@@ -340,13 +542,15 @@ class LabelledData(param.Parameterized):
         Assign a new label and/or group to an existing LabelledData
         object, creating a clone of the object with the new settings.
         """
-        keywords = [('label',label), ('group',group)]
-        obj = self.clone(self.data,
-                         **{k:v for k,v in keywords if v is not None})
-        if (depth > 0) and getattr(obj, '_deep_indexable', False):
-            for k, v in obj.items():
-                obj[k] =  v.relabel(group=group, label=label, depth=depth-1)
-        return obj
+        new_data = self.data
+        if (depth > 0) and getattr(self, '_deep_indexable', False):
+            new_data = []
+            for k, v in self.data.items():
+                relabelled = v.relabel(group=group, label=label, depth=depth-1)
+                new_data.append((k, relabelled))
+        keywords = [('label', label), ('group', group)]
+        kwargs = {k: v for k, v in keywords if v is not None}
+        return self.clone(new_data, **kwargs)
 
 
     def matches(self, spec):
@@ -403,6 +607,8 @@ class LabelledData(param.Parameterized):
         # Assumes composite objects are iterables
         if self._deep_indexable:
             for el in self:
+                if el is None:
+                    continue
                 accumulator += el.traverse(fn, specs, full_breadth)
                 if not full_breadth: break
         return accumulator
@@ -413,6 +619,7 @@ class LabelledData(param.Parameterized):
         Recursively replaces elements using a map function when the
         specification applies.
         """
+        if specs and not isinstance(specs, list): specs = [specs]
         applies = specs is None or any(self.matches(spec) for spec in specs)
 
         if self._deep_indexable:
@@ -558,7 +765,6 @@ class Dimensioned(LabelledData):
        A string describing the data wrapped by the object.""")
 
     __abstract = True
-    _sorted = False
     _dim_groups = ['kdims', 'vdims', 'cdims', 'ddims']
     _dim_aliases = dict(key_dimensions='kdims', value_dimensions='vdims',
                         constant_dimensions='cdims', deep_dimensions='ddims')
@@ -597,11 +803,12 @@ class Dimensioned(LabelledData):
         cdims = [(d.name, val) for d, val in self.cdims.items()]
         self._cached_constants = OrderedDict(cdims)
         self._settings = None
+        self.redim = redim(self)
 
 
     def _valid_dimensions(self, dimensions):
         """Validates key dimension input
-        
+
         Returns kdims if no dimensions are specified"""
         if dimensions is None:
             dimensions = self.kdims
@@ -620,7 +827,7 @@ class Dimensioned(LabelledData):
     @property
     def ddims(self):
         "The list of deep dimensions"
-        if self._deep_indexable and len(self):
+        if self._deep_indexable and self:
             return self.values()[0].dimensions()
         else:
             return []
@@ -633,6 +840,13 @@ class Dimensioned(LabelledData):
         by their type, i.e. 'key' or 'value' dimensions.
         By default 'all' dimensions are returned.
         """
+        if label in ['name', True]:
+            label = 'short'
+        elif label == 'label':
+            label = 'long'
+        elif label:
+            raise ValueError("label needs to be one of True, False, 'name' or 'label'")
+
         lambdas = {'k': (lambda x: x.kdims, {'full_breadth': False}),
                    'v': (lambda x: x.vdims, {}),
                    'c': (lambda x: x.cdims, {})}
@@ -652,7 +866,8 @@ class Dimensioned(LabelledData):
         else:
             raise KeyError("Invalid selection %r, valid selections include"
                            "'all', 'value' and 'key' dimensions" % repr(selection))
-        return [dim.name if label else dim for dim in dims]
+        return [(dim.label if label == 'long' else dim.name)
+                if label else dim for dim in dims]
 
 
     def get_dimension(self, dimension, default=None, strict=False):
@@ -673,6 +888,8 @@ class Dimensioned(LabelledData):
             else:
                 return default
         name_map = {dim.name: dim for dim in all_dims}
+        name_map.update({dim.label: dim for dim in all_dims})
+        name_map.update({dimension_sanitizer(dim.name): dim for dim in all_dims})
         if strict and dimension not in name_map:
             raise KeyError("Dimension %s not found" % dimension)
         else:
@@ -691,10 +908,9 @@ class Dimensioned(LabelledData):
             else:
                 return IndexError('Dimension index out of bounds')
         try:
-            if dim in self.kdims+self.vdims:
-                return (self.kdims+self.vdims).index(dim)
-            return self.dimensions().index(dim)
-        except ValueError:
+            dimensions = self.kdims+self.vdims
+            return [i for i, d in enumerate(dimensions) if d == dim][0]
+        except IndexError:
             raise Exception("Dimension %s not found in %s." %
                             (dim, self.__class__.__name__))
 
@@ -751,6 +967,8 @@ class Dimensioned(LabelledData):
 
         # Check selection_spec applies
         if selection_specs is not None:
+            if not isinstance(selection_specs, (list, tuple)):
+                selection_specs = [selection_specs]
             matches = any(self.matches(spec)
                           for spec in selection_specs)
         else:
@@ -758,8 +976,9 @@ class Dimensioned(LabelledData):
 
         # Apply selection to self
         if local_kwargs and matches:
-            ndims = (len(self.dimensions()) if any(d in self.vdims for d in kwargs)
-                     else self.ndims)
+            ndims = self.ndims
+            if any(d in self.vdims for d in kwargs):
+                ndims = len(self.kdims+self.vdims)
             select = [slice(None) for _ in range(ndims)]
             for dim, val in local_kwargs.items():
                 if dim == 'value':
@@ -780,20 +999,15 @@ class Dimensioned(LabelledData):
             return selection
         elif type(selection) is not type(self) and isinstance(selection, Dimensioned):
             # Apply the selection on the selected object of a different type
-            val_dim = ['value'] if selection.vdims else []
-            key_dims = selection.dimensions('key', label=True) + val_dim
-            if any(kw in key_dims for kw in kwargs):
+            dimensions = selection.dimensions() + ['value']
+            if any(kw in dimensions for kw in kwargs):
                 selection = selection.select(selection_specs, **kwargs)
         elif isinstance(selection, Dimensioned) and selection._deep_indexable:
             # Apply the deep selection on each item in local selection
             items = []
             for k, v in selection.items():
-                val_dim = ['value'] if v.vdims else []
-                dims = list(zip(*[(dimension_sanitizer(kd), kd)
-                                  for kd in v.dimensions('key', label=True)]))
-                kdims, skdims = dims if dims else ([], [])
-                key_dims = list(kdims) + list(skdims) + val_dim
-                if any(kw in key_dims for kw in kwargs):
+                dimensions = v.dimensions() + ['value']
+                if any(kw in dimensions for kw in kwargs):
                     items.append((k, v.select(selection_specs, **kwargs)))
                 else:
                     items.append((k, v))
@@ -801,35 +1015,6 @@ class Dimensioned(LabelledData):
         return selection
 
 
-    def redim(self, specs=None, **dimensions):
-        """
-        Replaces existing dimensions in an object with new dimensions
-        or changing specific attributes of a dimensions. Dimension
-        mapping should map between the old dimension name and a
-        dictionary of the new attributes, a completely new dimension
-        or a new string name.
-        """
-        if specs is None:
-            applies = True
-        else:
-            if not isinstance(specs, list):
-                specs = [specs]
-            applies = any(self.matches(spec) for spec in specs)
-
-        redimmed = self
-        if self._deep_indexable:
-            deep_mapped = [(k, v.redim(specs, **dimensions))
-                           for k, v in self.items()]
-            redimmed = self.clone(deep_mapped)
-
-        if applies:
-            kdims = replace_dimensions(self.kdims, dimensions)
-            vdims = replace_dimensions(self.vdims, dimensions)
-            return redimmed.clone(kdims=kdims, vdims=vdims)
-        else:
-            return redimmed
-
-
     def dimension_values(self, dimension, expanded=True, flat=True):
         """
         Returns the values along the specified dimension. This method
@@ -854,21 +1039,30 @@ class Dimensioned(LabelledData):
         dimension = self.get_dimension(dimension)
         if dimension is None:
             return (None, None)
-        if dimension.range != (None, None):
+        if None not in dimension.range:
             return dimension.range
-        elif not data_range:
-            return (None, None)
-        soft_range = [r for r in dimension.soft_range
-                      if r is not None]
-        if dimension in self.kdims or dimension in self.vdims:
-            dim_vals = self.dimension_values(dimension.name)
-            return find_range(dim_vals, soft_range)
-        dname = dimension.name
-        match_fn = lambda x: dname in x.dimensions(['key', 'value'], True)
-        range_fn = lambda x: x.range(dname)
-        ranges = self.traverse(range_fn, [match_fn])
-        drange = max_range(ranges)
-        return drange
+        elif data_range:
+            if dimension in self.kdims+self.vdims:
+                dim_vals = self.dimension_values(dimension.name)
+                drange = find_range(dim_vals)
+            else:
+                dname = dimension.name
+                match_fn = lambda x: dname in x.kdims + x.vdims
+                range_fn = lambda x: x.range(dname)
+                ranges = self.traverse(range_fn, [match_fn])
+                drange = max_range(ranges)
+            soft_range = [r for r in dimension.soft_range if r is not None]
+            if soft_range:
+                drange = max_range([drange, soft_range])
+        else:
+            drange = dimension.soft_range
+        if dimension.range[0] is not None:
+            return (dimension.range[0], drange[1])
+        elif dimension.range[1] is not None:
+            return (drange[0], dimension.range[1])
+        else:
+            return drange
+
 
     def __repr__(self):
         reprval = PrettyPrinter.pprint(self)
@@ -877,6 +1071,9 @@ class Dimensioned(LabelledData):
         else:
             return str(reprval)
 
+    def __str__(self):
+        return repr(self)
+
     def __unicode__(self):
         return unicode(PrettyPrinter.pprint(self))
 
