@@ -7,6 +7,7 @@ to act as supplementary elements.
 
 from functools import reduce
 from itertools import chain
+from collections import defaultdict, Counter
 
 import numpy as np
 
@@ -15,8 +16,7 @@ import param
 from .dimension import Dimension, Dimensioned, ViewableElement
 from .ndmapping import OrderedDict, NdMapping, UniformNdMapping
 from .tree import AttrTree
-from .util import (int_to_roman, sanitize_identifier, group_sanitizer,
-                   label_sanitizer, unique_array)
+from .util import (unique_array, get_path, make_path_unique, int_to_roman)
 from . import traversal
 
 
@@ -27,7 +27,7 @@ class Composable(object):
     """
 
     def __add__(self, obj):
-        return Layout.from_values(self) + Layout.from_values(obj)
+        return Layout.from_values([self, obj])
 
 
     def __lshift__(self, other):
@@ -98,6 +98,56 @@ class AdjointLayout(Dimensioned):
         super(AdjointLayout, self).__init__(data, **params)
 
 
+    def __mul__(self, other, reverse=False):
+        layer1 = other if reverse else self
+        layer2 = self if reverse else other
+        adjoined_items = []
+        if isinstance(layer1, AdjointLayout) and isinstance(layer2, AdjointLayout):
+            adjoined_items = []
+            adjoined_items.append(layer1.main*layer2.main)
+            if layer1.right is not None and layer2.right is not None:
+                if layer1.right.dimensions() == layer2.right.dimensions():
+                    adjoined_items.append(layer1.right*layer2.right)
+                else:
+                    adjoined_items += [layer1.right, layer2.right]
+            elif layer1.right is not None:
+                adjoined_items.append(layer1.right)
+            elif layer2.right is not None:
+                adjoined_items.append(layer2.right)
+
+            if layer1.top is not None and layer2.top is not None:
+                if layer1.top.dimensions() == layer2.top.dimensions():
+                    adjoined_items.append(layer1.top*layer2.top)
+                else:
+                    adjoined_items += [layer1.top, layer2.top]
+            elif layer1.top is not None:
+                adjoined_items.append(layer1.top)
+            elif layer2.top is not None:
+                adjoined_items.append(layer2.top)
+            if len(adjoined_items) > 3:
+                raise ValueError("AdjointLayouts could not be overlaid, "
+                                 "the dimensions of the adjoined plots "
+                                 "do not match and the AdjointLayout can "
+                                 "hold no more than two adjoined plots.")
+        elif isinstance(layer1, AdjointLayout):
+            adjoined_items = [layer1.data[o] for o in self.layout_order
+                              if o in layer1.data]
+            adjoined_items[0] = layer1.main * layer2
+        elif isinstance(layer2, AdjointLayout):
+            adjoined_items = [layer2.data[o] for o in self.layout_order
+                              if o in layer2.data]
+            adjoined_items[0] = layer1 * layer2.main
+
+        if adjoined_items:
+            return self.clone(adjoined_items)
+        else:
+            return NotImplemented
+
+
+    def __rmul__(self, other):
+        return self.__mul__(other, reverse=True)
+
+
     @property
     def group(self):
         if self.main and self.main.group != type(self.main).__name__:
@@ -212,7 +262,7 @@ class AdjointLayout(Dimensioned):
 
 
     def __add__(self, obj):
-        return Layout.from_values(self) + Layout.from_values(obj)
+        return Layout.from_values([self, obj])
 
 
     def __len__(self):
@@ -268,7 +318,7 @@ class NdLayout(UniformNdMapping):
 
 
     def __add__(self, obj):
-        return Layout.from_values(self) + Layout.from_values(obj)
+        return Layout.from_values([self, obj])
 
 
     @property
@@ -302,18 +352,22 @@ class Layout(AttrTree, Dimensioned):
     their row and column index in the layout.
 
     The maximum number of columns in such a layout may be controlled
-    with the cols method and the display policy is set with the
-    display method. A display policy of 'auto' may use the string repr
-    of the tree for large trees that would otherwise take a long time
-    to display wheras a policy of 'all' will always display all the
-    available leaves. The detailed settings for the 'auto' policy may
-    be set using the max_branches option of the %output magic.
+    with the cols method.
     """
 
     group = param.String(default='Layout', constant=True)
 
     _deep_indexable = True
 
+    def __init__(self, items=None, identifier=None, parent=None, **kwargs):
+        self.__dict__['_max_cols'] = 4
+        if items and all(isinstance(item, Dimensioned) for item in items):
+            items = self._process_items(items)
+        params = {p: kwargs.pop(p) for p in list(self.params().keys())+['id'] if p in kwargs}
+        AttrTree.__init__(self, items, identifier, parent, **kwargs)
+        Dimensioned.__init__(self, self.data, **params)
+
+
     @classmethod
     def collate(cls, data, kdims=None, key_dimensions=None):
         kdims = key_dimensions if (kdims is None) else kdims
@@ -329,75 +383,71 @@ class Layout(AttrTree, Dimensioned):
 
 
     @classmethod
-    def new_path(cls, path, item, paths, count):
-        sanitizers = [sanitize_identifier, group_sanitizer, label_sanitizer]
-        path = tuple(fn(p) for (p, fn) in zip(path, sanitizers))
-        while any(path[:i] in paths or path in [p[:i] for p in paths]
-                  for i in range(1,len(path)+1)):
-            path = path[:2]
-            pl = len(path)
-            if (pl == 1 and not item.label) or (pl == 2 and item.label):
-                new_path = path + (int_to_roman(count-1),)
-                if path in paths:
-                    paths[paths.index(path)] = new_path
-                path = path + (int_to_roman(count),)
-            else:
-                path = path[:-1] + (int_to_roman(count),)
-            count += 1
-        return path, count
-
-
-    @classmethod
-    def relabel_item_paths(cls, items):
-        """
-        Given a list of path items (list of tuples where each element
-        is a (path, element) pair), generate a new set of path items that
-        guarantees that no paths clash. This uses the element labels as
-        appropriate and automatically generates roman numeral
-        identifiers if necessary.
-        """
-        paths, path_items = [], []
-        count = 2
-        for path, item in items:
-            new_path, count = cls.new_path(path, item, paths, count)
-            new_path = tuple(''.join((p[0].upper(), p[1:])) for p in new_path)
-            path_items.append(item)
-            paths.append(new_path)
-        return list(zip(paths, path_items))
-
-
-    @classmethod
-    def _from_values(cls, val):
-        return reduce(lambda x,y: x+y, val).display('auto')
-
-    @classmethod
-    def from_values(cls, val):
+    def from_values(cls, vals):
         """
         Returns a Layout given a list (or tuple) of viewable
         elements or just a single viewable element.
         """
-        collection = isinstance(val, (list, tuple))
-        if type(val) is cls:
-            return val
-        elif collection and len(val)>1:
-            return cls._from_values(val)
-        elif collection:
-            val = val[0]
-        group = group_sanitizer(val.group)
-        group = ''.join([group[0].upper(), group[1:]])
-        label = label_sanitizer(val.label if val.label else 'I')
-        label = ''.join([label[0].upper(), label[1:]])
-        return cls(items=[((group, label), val)])
+        return cls(items=cls._process_items(vals))
 
 
-    def __init__(self, items=None, identifier=None, parent=None, **kwargs):
-        self.__dict__['_display'] = 'auto'
-        self.__dict__['_max_cols'] = 4
-        if items and all(isinstance(item, Dimensioned) for item in items):
-            items = self.from_values(items).data
-        params = {p: kwargs.pop(p) for p in list(self.params().keys())+['id'] if p in kwargs}
-        AttrTree.__init__(self, items, identifier, parent, **kwargs)
-        Dimensioned.__init__(self, self.data, **params)
+    @classmethod
+    def _process_items(cls, vals):
+        """
+        Processes a list of Labelled types unpacking any objects of
+        the same type (e.g. a Layout) and finding unique paths for
+        all the items in the list.
+        """
+        if type(vals) is cls:
+            return vals.data
+        elif not isinstance(vals, (list, tuple)):
+            vals = [vals]
+        items = []
+        counts = defaultdict(lambda: 1)
+        cls._unpack_paths(vals, items, counts)
+        items = cls._deduplicate_items(items)
+        return items
+
+
+    @classmethod
+    def _deduplicate_items(cls, items):
+        """
+        Iterates over the paths a second time and ensures that partial
+        paths are not overlapping.
+        """
+        counter = Counter([path[:i] for path, _ in items for i in range(1, len(path)+1)])
+        if sum(counter.values()) == len(counter):
+            return items
+
+        new_items = []
+        counts = defaultdict(lambda: 0)
+        for i, (path, item) in enumerate(items):
+            if counter[path] > 1:
+                path = path + (int_to_roman(counts[path]+1),)
+            elif counts[path]:
+                path = path[:-1] + (int_to_roman(counts[path]+1),)
+            new_items.append((path, item))
+            counts[path] += 1
+        return new_items
+
+
+    @classmethod
+    def _unpack_paths(cls, objs, items, counts):
+        """
+        Recursively unpacks lists and Layout-like objects, accumulating
+        into the supplied list of items.
+        """
+        if type(objs) is cls:
+            objs = objs.items()
+        for item in objs:
+            path, obj = item if isinstance(item, tuple) else (None, item)
+            if type(obj) is cls:
+                cls._unpack_paths(obj, items, counts)
+                continue
+            new = path is None or len(path) == 1
+            path = get_path(item) if new else path
+            new_path = make_path_unique(path, counts, new)
+            items.append((new_path, obj))
 
 
     @property
@@ -418,7 +468,6 @@ class Layout(AttrTree, Dimensioned):
         # Standard relabel method except _max_cols and _display transferred
         relabelled = super(Layout, self).relabel(label=label, group=group, depth=depth)
         relabelled.__dict__['_max_cols'] = self.__dict__['_max_cols']
-        relabelled.__dict__['_display'] = self.__dict__['_display']
         return relabelled
 
     def clone(self, *args, **overrides):
@@ -427,7 +476,6 @@ class Layout(AttrTree, Dimensioned):
         display mode is also propagated.
         """
         clone = super(Layout, self).clone(*args, **overrides)
-        clone._display = self._display
         clone._max_cols = self._max_cols
         clone.id = self.id
         return clone
@@ -454,16 +502,12 @@ class Layout(AttrTree, Dimensioned):
 
     def display(self, option):
         "Sets the display policy of the Layout before returning self"
-        options = ['auto', 'all']
-        if option not in options:
-            raise Exception("Display option must be one of %s" %
-                            ','.join(repr(el) for el in options))
-        self._display = option
+        self.warning('Layout display option is deprecated and no longer needs to be used')
         return self
 
 
     def select(self, selection_specs=None, **selections):
-        return super(Layout, self).select(selection_specs, **selections).display(self._display)
+        return super(Layout, self).select(selection_specs, **selections)
 
 
     def grid_items(self):
@@ -485,6 +529,9 @@ class Layout(AttrTree, Dimensioned):
             if key < len(self):
                 return self.data.values()[key]
             raise KeyError("Element out of range.")
+        elif isinstance(key, slice):
+            raise KeyError("A Layout may not be sliced, ensure that you "
+                           "are slicing on a leaf (i.e. not a branch) of the Layout.")
         if len(key) == 2 and not any([isinstance(k, str) for k in key]):
             if key == (slice(None), slice(None)): return self
             row, col = key
@@ -501,9 +548,7 @@ class Layout(AttrTree, Dimensioned):
 
 
     def __add__(self, other):
-        other = self.from_values(other)
-        items = list(self.data.items()) + list(other.data.items())
-        return Layout(items=self.relabel_item_paths(items)).display('all')
+        return Layout.from_values([self, other])
 
 
 
