@@ -1,21 +1,30 @@
 import os, sys, warnings, operator
+import types
 import numbers
+import inspect
 import itertools
 import string, fnmatch
 import unicodedata
 import datetime as dt
 from collections import defaultdict
+from functools import partial
+from contextlib import contextmanager
 
 import numpy as np
 import param
 
+import json
+
 try:
     from cyordereddict import OrderedDict
 except:
     from collections import OrderedDict
 
+datetime_types = (np.datetime64, dt.datetime)
+
 try:
     import pandas as pd # noqa (optional import)
+    datetime_types = datetime_types + (pd.tslib.Timestamp,)
 except ImportError:
     pd = None
 
@@ -24,8 +33,64 @@ try:
 except ImportError:
     dd = None
 
+
+
+
+class HashableJSON(json.JSONEncoder):
+    """
+    Extends JSONEncoder to generate a hashable string for as many types
+    of object as possible including nested objects and objects that are
+    not normally hashable. The purpose of this class is to generate
+    unique strings that once hashed are suitable for use in memoization
+    and other cases where deep equality must be tested without storing
+    the entire object.
+
+    By default JSONEncoder supports booleans, numbers, strings, lists,
+    tuples and dictionaries. In order to support other types such as
+    sets, datetime objects and mutable objects such as pandas Dataframes
+    or numpy arrays, HashableJSON has to convert these types to
+    datastructures that can normally be represented as JSON.
+
+    Support for other object types may need to be introduced in
+    future. By default, unrecognized object types are represented by
+    their id.
+
+    One limitation of this approach is that dictionaries with composite
+    keys (e.g tuples) are not supported due to the JSON spec.
+    """
+    string_hashable = (dt.datetime,)
+    repr_hashable = ()
+
+    def default(self, obj):
+        if isinstance(obj, set):
+            return hash(frozenset(obj))
+        elif isinstance(obj, np.ndarray):
+            return obj.tolist()
+        if pd and isinstance(obj, (pd.Series, pd.DataFrame)):
+            return repr(sorted(list(obj.to_dict().items())))
+        elif isinstance(obj, self.string_hashable):
+            return str(obj)
+        elif isinstance(obj, self.repr_hashable):
+            return repr(obj)
+        try:
+            return hash(obj)
+        except:
+            return id(obj)
+
+
+
+def deephash(obj):
+    """
+    Given an object, return a hash using HashableJSON. This hash is not
+    architecture, Python version or platform independent.
+    """
+    try:
+        return hash(json.dumps(obj, cls=HashableJSON, sort_keys=True))
+    except:
+        return None
+
+
 # Python3 compatibility
-import types
 if sys.version_info.major == 3:
     basestring = str
     unicode = str
@@ -37,6 +102,135 @@ else:
     generator_types = (izip, xrange, types.GeneratorType)
 
 
+
+def argspec(callable_obj):
+    """
+    Returns an ArgSpec object for functions, staticmethods, instance
+    methods, classmethods and partials.
+
+    Note that the args list for instance and class methods are those as
+    seen by the user. In other words, the first argument which is
+    conventionally called 'self' or 'cls' is omitted in these cases.
+    """
+    if (isinstance(callable_obj, type)
+        and issubclass(callable_obj, param.ParameterizedFunction)):
+        # Parameterized function.__call__ considered function in py3 but not py2
+        spec = inspect.getargspec(callable_obj.__call__)
+        args=spec.args[1:]
+    elif inspect.isfunction(callable_obj):    # functions and staticmethods
+        return inspect.getargspec(callable_obj)
+    elif isinstance(callable_obj, partial): # partials
+        arglen = len(callable_obj.args)
+        spec =  inspect.getargspec(callable_obj.func)
+        args = [arg for arg in spec.args[arglen:] if arg not in callable_obj.keywords]
+    elif inspect.ismethod(callable_obj):    # instance and class methods
+        spec = inspect.getargspec(callable_obj)
+        args = spec.args[1:]
+    else:                                   # callable objects
+        return argspec(callable_obj.__call__)
+
+    return inspect.ArgSpec(args     = args,
+                           varargs  = spec.varargs,
+                           keywords = spec.keywords,
+                           defaults = spec.defaults)
+
+
+
+def validate_dynamic_argspec(argspec, kdims, streams):
+    """
+    Utility used by DynamicMap to ensure the supplied callback has an
+    appropriate signature.
+
+    If validation succeeds, returns a list of strings to be zipped with
+    the positional arguments i.e kdim values. The zipped values can then
+    be merged with the stream values to pass everything to the Callable
+    as keywords.
+
+    If the callbacks use *args, None is returned to indicate that kdim
+    values must be passed to the Callable by position. In this
+    situation, Callable passes *args and **kwargs directly to the
+    callback.
+
+    If the callback doesn't use **kwargs, the accepted keywords are
+    validated against the stream parameter names.
+    """
+
+    kdims = [kdim.name for kdim in kdims]
+    stream_params = stream_parameters(streams)
+    defaults = argspec.defaults if argspec.defaults else []
+    all_posargs = argspec.args[:-len(defaults)] if defaults else argspec.args
+    # Filter out any posargs for streams
+    posargs = [arg for arg in all_posargs if arg not in stream_params]
+    kwargs = argspec.args[-len(defaults):]
+
+    if argspec.keywords is None:
+        unassigned_streams = set(stream_params) - set(argspec.args)
+        if unassigned_streams:
+            raise KeyError('Callable missing keywords to accept %s stream parameters'
+                           % ', '.join(unassigned_streams))
+
+
+    if len(posargs) > len(kdims) + len(stream_params):
+        raise KeyError('Callable accepts more positional arguments than '
+                       'there are kdims and stream parameters')
+    if kdims == []:                  # Can be no posargs, stream kwargs already validated
+        return []
+    if set(kdims) == set(posargs):   # Posargs match exactly, can all be passed as kwargs
+        return kdims
+    elif len(posargs) == len(kdims): # Posargs match kdims length, supplying names
+        if argspec.args[:len(kdims)] != posargs:
+            raise KeyError('Unmatched positional kdim arguments only '
+                           'allowed at the start of the signature')
+
+        return posargs
+    elif argspec.varargs:            # Posargs missing, passed to Callable directly
+        return None
+    elif set(posargs) - set(kdims):
+        raise KeyError('Callable accepts more positional arguments {posargs} '
+                       'than there are key dimensions {kdims}'.format(posargs=posargs,
+                                                                      kdims=kdims))
+    elif set(kdims).issubset(set(kwargs)): # Key dims can be supplied by keyword
+        return kdims
+    elif set(kdims).issubset(set(posargs+kwargs)):
+        return kdims
+    else:
+        raise KeyError('Callback signature over {names} does not accommodate '
+                       'required kdims {kdims}'.format(names=list(set(posargs+kwargs)),
+                                                       kdims=kdims))
+
+
+def callable_name(callable_obj):
+    """
+    Attempt to return a meaningful name identifying a callable or generator
+    """
+    try:
+        if (isinstance(callable_obj, type)
+            and issubclass(callable_obj, param.ParameterizedFunction)):
+            return callable_obj.__name__
+        elif (isinstance(callable_obj, param.Parameterized)
+              and 'operation' in callable_obj.params()):
+            return callable_obj.operation.__name__
+        elif isinstance(callable_obj, partial):
+            return str(callable_obj)
+        elif inspect.isfunction(callable_obj):  # functions and staticmethods
+            return callable_obj.__name__
+        elif inspect.ismethod(callable_obj):    # instance and class methods
+            meth = callable_obj
+            if sys.version_info < (3,0):
+                owner =  meth.im_class if meth.im_self is None else meth.im_self
+            else:
+                owner =  meth.__self__
+            if meth.__name__ == '__call__':
+                return type(owner).__name__
+            return '.'.join([owner.__name__, meth.__name__])
+        elif isinstance(callable_obj, types.GeneratorType):
+            return callable_obj.__name__
+        else:
+            return type(callable_obj).__name__
+    except:
+        return str(callable_obj)
+
+
 def process_ellipses(obj, key, vdim_selection=False):
     """
     Helper function to pad a __getitem__ key with the right number of
@@ -68,10 +262,13 @@ def process_ellipses(obj, key, vdim_selection=False):
     return head + ((slice(None),) * padlen) + tail
 
 
-def safe_unicode(value):
-   if sys.version_info.major == 3 or not isinstance(value, str): return value
-   else: return unicode(value.decode('utf-8'))
-
+def bytes_to_unicode(value):
+    """
+    Safely casts bytestring to unicode
+    """
+    if isinstance(value, bytes):
+        return unicode(value.decode('utf-8'))
+    return value
 
 
 def capitalize_unicode_name(s):
@@ -160,7 +357,7 @@ class sanitize_identifier_fn(param.ParameterizedFunction):
     transforms = param.List(default=[capitalize_unicode_name], doc="""
        List of string transformation functions to apply after
        filtering and substitution in order to further compress the
-       unicode name. For instance, the defaultcapitalize_unicode_name
+       unicode name. For instance, the default capitalize_unicode_name
        function will turn the string "capital delta" into "Delta".""")
 
     disallowed = param.List(default=['trait_names', '_ipython_display_',
@@ -235,7 +432,7 @@ class sanitize_identifier_fn(param.ParameterizedFunction):
         for c in identifier:
             replacement = unicodedata.normalize('NFKD', c).encode('ASCII', 'ignore')
             if replacement != '':
-                chars += safe_unicode(replacement)
+                chars += bytes_to_unicode(replacement)
             else:
                 chars += c
         return chars
@@ -266,7 +463,7 @@ class sanitize_identifier_fn(param.ParameterizedFunction):
             return self.aliases[name]
         elif name in self._lookup_table:
            return self._lookup_table[name]
-        name = safe_unicode(name)
+        name = bytes_to_unicode(name)
         version = self.version if version is None else version
         if not self.allowable(name):
             raise AttributeError("String %r is in the disallowed list of attribute names: %r" % self.disallowed)
@@ -399,6 +596,9 @@ def max_range(ranges):
             if pd and all(isinstance(v, pd.tslib.Timestamp) for r in values for v in r):
                 values = [(v1.to_datetime64(), v2.to_datetime64()) for v1, v2 in values]
             arr = np.array(values)
+            if arr.dtype.kind in 'OSU':
+                arr = np.sort([v for v in arr.flat if not is_nan(v)])
+                return arr[0], arr[-1]
             if arr.dtype.kind in 'M':
                 return arr[:, 0].min(), arr[:, 1].max()
             return (np.nanmin(arr[:, 0]), np.nanmax(arr[:, 1]))
@@ -429,14 +629,18 @@ def max_extents(extents, zrange=False):
     with warnings.catch_warnings():
         warnings.filterwarnings('ignore', r'All-NaN (slice|axis) encountered')
         for lidx, uidx in inds:
-            lower = [v for v in arr[lidx] if v is not None]
-            upper = [v for v in arr[uidx] if v is not None]
-            if lower and isinstance(lower[0], np.datetime64):
+            lower = [v for v in arr[lidx] if v is not None and not is_nan(v)]
+            upper = [v for v in arr[uidx] if v is not None and not is_nan(v)]
+            if lower and isinstance(lower[0], datetime_types):
                 extents[lidx] = np.min(lower)
+            elif any(isinstance(l, basestring) for l in lower):
+                extents[lidx] = np.sort(lower)[0]
             elif lower:
                 extents[lidx] = np.nanmin(lower)
-            if upper and isinstance(upper[0], np.datetime64):
+            if upper and isinstance(upper[0], datetime_types):
                 extents[uidx] = np.max(upper)
+            elif any(isinstance(u, basestring) for u in upper):
+                extents[uidx] = np.sort(upper)[-1]
             elif upper:
                 extents[uidx] = np.nanmax(upper)
     return tuple(extents)
@@ -494,7 +698,9 @@ def unique_array(arr):
     """
     Returns an array of unique values in the input order
     """
-    if pd:
+    if not len(arr):
+        return arr
+    elif pd:
         return pd.unique(arr)
     else:
         _, uniq_inds = np.unique(arr, return_index=True)
@@ -537,6 +743,32 @@ def python2sort(x,key=None):
     return itertools.chain.from_iterable(sorted(group, key=key) for group in groups)
 
 
+def merge_dimensions(dimensions_list):
+    """
+    Merges lists of fully or partially overlapping dimensions by
+    merging their values.
+
+    >>> from holoviews import Dimension
+    >>> dim_list = [[Dimension('A', values=[1, 2, 3]), Dimension('B')],
+    ...             [Dimension('A', values=[2, 3, 4])]]
+    >>> dimensions = merge_dimensions(dim_list)
+    >>> dimensions
+    [Dimension('A'), Dimension('B')]
+    >>> dimensions[0].values
+    [1, 2, 3, 4]
+    """
+    dvalues = defaultdict(list)
+    dimensions = []
+    for dims in dimensions_list:
+        for d in dims:
+            dvalues[d.name].append(d.values)
+            if d not in dimensions:
+                dimensions.append(d)
+    dvalues = {k: list(unique_iterator(itertools.chain(*vals)))
+               for k, vals in dvalues.items()}
+    return [d(values=dvalues.get(d.name, [])) for d in dimensions]
+
+
 def dimension_sort(odict, kdims, vdims, categorical, key_index, cached_values):
     """
     Sorts data by key using usual Python tuple sorting semantics
@@ -548,6 +780,7 @@ def dimension_sort(odict, kdims, vdims, categorical, key_index, cached_values):
     indexes = [(dimensions[i], int(i not in range(ndims)),
                     i if i in range(ndims) else i-ndims)
                 for i in key_index]
+    cached_values = {d: [None]+vals for d, vals in cached_values.items()}
 
     if len(set(key_index)) != len(key_index):
         raise ValueError("Cannot sort on duplicated dimensions")
@@ -605,14 +838,14 @@ def sort_topologically(graph):
     }
 
     sort_topologically(graph)
-    [set([1, 2]), set([3, 4]), set([5, 6])]
+    [[1, 2], [3, 4], [5, 6]]
     """
     levels_by_name = {}
-    names_by_level = defaultdict(set)
+    names_by_level = defaultdict(list)
 
     def add_level_to_name(name, level):
         levels_by_name[name] = level
-        names_by_level[level].add(name)
+        names_by_level[level].append(name)
 
 
     def walk_depth_first(name):
@@ -645,6 +878,35 @@ def sort_topologically(graph):
                                     (names_by_level.get(i, None)
                                      for i in itertools.count())))
 
+
+def is_cyclic(graph):
+    """
+    Return True if the directed graph g has a cycle. The directed graph
+    should be represented as adictionary mapping of edges for each node.
+    """
+    path = set()
+
+    def visit(vertex):
+        path.add(vertex)
+        for neighbour in graph.get(vertex, ()):
+            if neighbour in path or visit(neighbour):
+                return True
+        path.remove(vertex)
+        return False
+
+    return any(visit(v) for v in graph)
+
+
+def one_to_one(graph, nodes):
+    """
+    Return True if graph contains only one to one mappings. The
+    directed graph should be represented as a dictionary mapping of
+    edges for each node. Nodes should be passed a simple list.
+    """
+    edges = itertools.chain.from_iterable(graph.values())
+    return len(graph) == len(nodes) and len(set(edges)) == len(nodes)
+
+
 def get_overlay_spec(o, k, v):
     """
     Gets the type.group.label + key spec from an Element in an Overlay.
@@ -727,12 +989,17 @@ def get_spec(obj):
 
 def find_file(folder, filename):
     """
-    Find a file given folder and filename.
+    Find a file given folder and filename. If the filename can be
+    resolved directly returns otherwise walks the supplied folder.
     """
     matches = []
+    if os.path.isabs(filename) and os.path.isfile(filename):
+        return filename
     for root, _, filenames in os.walk(folder):
-        for filename in fnmatch.filter(filenames, filename):
-            matches.append(os.path.join(root, filename))
+        for fn in fnmatch.filter(filenames, filename):
+            matches.append(os.path.join(root, fn))
+    if not matches:
+        raise IOError('File %s could not be found' % filename)
     return matches[-1]
 
 
@@ -747,11 +1014,31 @@ def is_dataframe(data):
 def get_param_values(data):
     params = dict(kdims=data.kdims, vdims=data.vdims,
                   label=data.label)
-    if data.group != data.params()['group'].default:
+    if (data.group != data.params()['group'].default and not
+        isinstance(type(data).group, property)):
         params['group'] = data.group
     return params
 
 
+@contextmanager
+def disable_constant(parameterized):
+    """
+    Temporarily set parameters on Parameterized object to
+    constant=False.
+    """
+    params = parameterized.params().values()
+    constants = [p.constant for p in params]
+    for p in params:
+        p.constant = False
+    try:
+        yield
+    except:
+        raise
+    finally:
+        for (p, const) in zip(params, constants):
+            p.constant = const
+
+
 def get_ndmapping_label(ndmapping, attr):
     """
     Function to get the first non-auxiliary object
@@ -778,6 +1065,109 @@ def wrap_tuple(unwrapped):
     return (unwrapped if isinstance(unwrapped, tuple) else (unwrapped,))
 
 
+def stream_name_mapping(stream, exclude_params=['name'], reverse=False):
+    """
+    Return a complete dictionary mapping between stream parameter names
+    to their applicable renames, excluding parameters listed in
+    exclude_params.
+
+    If reverse is True, the mapping is from the renamed strings to the
+    original stream parameter names.
+    """
+    filtered = [k for k in stream.params().keys() if k not in exclude_params]
+    mapping = {k:stream._rename.get(k,k) for k in filtered}
+    if reverse:
+        return {v:k for k,v in mapping.items()}
+    else:
+        return mapping
+
+def rename_stream_kwargs(stream, kwargs, reverse=False):
+    """
+    Given a stream and a kwargs dictionary of parameter values, map to
+    the corresponding dictionary where the keys are substituted with the
+    appropriately renamed string.
+
+    If reverse, the output will be a dictionary using the original
+    parameter names given a dictionary using the renamed equivalents.
+    """
+    mapped_kwargs = {}
+    mapping = stream_name_mapping(stream, reverse=reverse)
+    for k,v in kwargs.items():
+        if k not in mapping:
+            msg = 'Could not map key {key} {direction} renamed equivalent'
+            direction = 'from' if reverse else 'to'
+            raise KeyError(msg.format(key=repr(k), direction=direction))
+        mapped_kwargs[mapping[k]] = v
+    return mapped_kwargs
+
+
+def stream_parameters(streams, no_duplicates=True, exclude=['name']):
+    """
+    Given a list of streams, return a flat list of parameter name,
+    excluding those listed in the exclude list.
+
+    If no_duplicates is enabled, a KeyError will be raised if there are
+    parameter name clashes across the streams.
+    """
+    param_groups = [s.contents.keys() for s in streams]
+    names = [name for group in param_groups for name in group]
+
+    if no_duplicates:
+        clashes = sorted(set([n for n in names if names.count(n) > 1]))
+        clash_streams = [s for s in streams for c in clashes if c in s.contents]
+        if clashes:
+            clashing = ', '.join([repr(c) for c in clash_streams[:-1]])
+            raise Exception('The supplied stream objects %s and %s '
+                            'clash on the following parameters: %r'
+                            % (clashing, clash_streams[-1], clashes))
+    return [name for name in names if name not in exclude]
+
+
+def dimensionless_contents(streams, kdims, no_duplicates=True):
+    """
+    Return a list of stream parameters that have not been associated
+    with any of the key dimensions.
+    """
+    names = stream_parameters(streams, no_duplicates)
+    return [name for name in names if name not in kdims]
+
+
+def unbound_dimensions(streams, kdims, no_duplicates=True):
+    """
+    Return a list of dimensions that have not been associated with
+    any streams.
+    """
+    params = stream_parameters(streams, no_duplicates)
+    return [d for d in kdims if d not in params]
+
+
+def wrap_tuple_streams(unwrapped, kdims, streams):
+    """
+    Fills in tuple keys with dimensioned stream values as appropriate.
+    """
+    param_groups = [(s.contents.keys(), s) for s in streams]
+    pairs = [(name,s)  for (group, s) in param_groups for name in group]
+    substituted = []
+    for pos,el in enumerate(wrap_tuple(unwrapped)):
+        if el is None and pos < len(kdims):
+            matches = [(name,s) for (name,s) in pairs if name==kdims[pos].name]
+            if len(matches) == 1:
+                (name, stream) = matches[0]
+                el = stream.contents[name]
+        substituted.append(el)
+    return tuple(substituted)
+
+
+def drop_streams(streams, kdims, keys):
+    """
+    Drop any dimensionsed streams from the keys and kdims.
+    """
+    stream_params = stream_parameters(streams)
+    inds, dims = zip(*[(ind, kdim) for ind, kdim in enumerate(kdims)
+                       if kdim not in stream_params])
+    return dims, [tuple(wrap_tuple(key)[ind] for ind in inds) for key in keys]
+
+
 def itervalues(obj):
     "Get value iterator from dictionary for Python 2 and 3"
     return iter(obj.values()) if sys.version_info.major == 3 else obj.itervalues()
@@ -806,6 +1196,53 @@ def unpack_group(group, getter):
             yield (wrap_tuple(key), obj)
 
 
+def capitalize(string):
+    """
+    Capitalizes the first letter of a string.
+    """
+    return string[0].upper() + string[1:]
+
+
+def get_path(item):
+    """
+    Gets a path from an Labelled object or from a tuple of an existing
+    path and a labelled object. The path strings are sanitized and
+    capitalized.
+    """
+    sanitizers = [group_sanitizer, label_sanitizer]
+    if isinstance(item, tuple):
+        path, item = item
+        if item.label:
+            if len(path) > 1 and item.label == path[1]:
+                path = path[:2]
+            else:
+                path = path[:1] + (item.label,)
+        else:
+            path = path[:1]
+    else:
+        path = (item.group, item.label) if item.label else (item.group,)
+    return tuple(capitalize(fn(p)) for (p, fn) in zip(path, sanitizers))
+
+
+def make_path_unique(path, counts, new):
+    """
+    Given a path, a list of existing paths and counts for each of the
+    existing paths.
+    """
+    added = False
+    while any(path == c[:i] for c in counts for i in range(1, len(c)+1)):
+        count = counts[path]
+        counts[path] += 1
+        if (not new and len(path) > 1) or added:
+            path = path[:-1]
+        else:
+            added = True
+        path = path + (int_to_roman(count),)
+    if len(path) == 1:
+        path = path + (int_to_roman(counts.get(path, 1)),)
+    if path not in counts:
+        counts[path] = 1
+    return path
 
 
 class ndmapping_groupby(param.ParameterizedFunction):
@@ -861,13 +1298,17 @@ class ndmapping_groupby(param.ParameterizedFunction):
         return container_type(groups, kdims=dimensions)
 
 
-def cartesian_product(arrays):
+def cartesian_product(arrays, flat=True, copy=False):
     """
-    Computes the cartesian product of a list of 1D arrays
-    returning arrays matching the shape defined by all
-    supplied dimensions.
+    Efficient cartesian product of a list of 1D arrays returning the
+    expanded array views for each dimensions. By default arrays are
+    flattened, which may be controlled with the flat flag. The array
+    views can be turned into regular arrays with the copy flag.
     """
-    return np.broadcast_arrays(*np.ix_(*arrays))
+    arrays = np.broadcast_arrays(*np.ix_(*arrays))
+    if flat:
+        return tuple(arr.flatten() if copy else arr.flat for arr in arrays)
+    return tuple(arr.copy() if copy else arr for arr in arrays)
 
 
 def arglexsort(arrays):
@@ -882,29 +1323,17 @@ def arglexsort(arrays):
     return recarray.argsort()
 
 
-def get_dynamic_item(map_obj, dimensions, key):
+def dimensioned_streams(dmap):
     """
-    Looks up an item in a DynamicMap given a list of dimensions
-    and a corresponding key. The dimensions must be a subset
-    of the map_obj key dimensions.
+    Given a DynamicMap return all streams that have any dimensioned
+    parameters i.e parameters also listed in the key dimensions.
     """
-    if isinstance(key, tuple):
-        dims = {d.name: k for d, k in zip(dimensions, key)
-                if d in map_obj.kdims}
-        key = tuple(dims.get(d.name) for d in map_obj.kdims)
-        el = map_obj.select([lambda x: type(x).__name__ == 'DynamicMap'],
-                            **dims)
-    elif key < map_obj.counter:
-        key_offset = max([key-map_obj.cache_size, 0])
-        key = map_obj.keys()[min([key-key_offset,
-                                  len(map_obj)-1])]
-        el = map_obj[key]
-    elif key >= map_obj.counter:
-        el = next(map_obj)
-        key = list(map_obj.keys())[-1]
-    else:
-        el = None
-    return key, el
+    dimensioned = []
+    for stream in dmap.streams:
+        stream_params = stream_parameters([stream])
+        if set([str(k) for k in dmap.kdims]) & set(stream_params):
+            dimensioned.append(stream)
+    return dimensioned
 
 
 def expand_grid_coords(dataset, dim):
@@ -916,7 +1345,7 @@ def expand_grid_coords(dataset, dim):
     arrays = [dataset.interface.coords(dataset, d.name, True)
               for d in dataset.kdims]
     idx = dataset.get_dimension_index(dim)
-    return cartesian_product(arrays)[idx]
+    return cartesian_product(arrays, flat=False)[idx]
 
 
 def dt64_to_dt(dt64):
@@ -925,3 +1354,29 @@ def dt64_to_dt(dt64):
     """
     ts = (dt64 - np.datetime64('1970-01-01T00:00:00Z')) / np.timedelta64(1, 's')
     return dt.datetime.utcfromtimestamp(ts)
+
+
+def is_nan(x):
+    """
+    Checks whether value is NaN on arbitrary types
+    """
+    try:
+        return np.isnan(x)
+    except:
+        return False
+
+
+def bound_range(vals, density):
+    """
+    Computes a bounding range and density from a number of samples
+    assumed to be evenly spaced. Density is rounded to machine precision
+    using significant digits reported by sys.float_info.dig.
+    """
+    low, high = vals.min(), vals.max()
+    invert = False
+    if vals[0] > vals[1]:
+        invert = True
+    if not density:
+        density = round(1./((high-low)/(len(vals)-1)), sys.float_info.dig)
+    halfd = 0.5/density
+    return low-halfd, high+halfd, density, invert
