@@ -25,7 +25,7 @@ from numpy.testing import assert_array_equal, assert_array_almost_equal
 from . import *    # noqa (All Elements need to support comparison)
 from ..core import (Element, Empty, AdjointLayout, Overlay, Dimension,
                     HoloMap, Dimensioned, Layout, NdLayout, NdOverlay,
-                    GridSpace, DynamicMap, GridMatrix)
+                    GridSpace, DynamicMap, GridMatrix, OrderedDict)
 from ..core.options import Options, Cycle
 from ..interface.pandas import DFrame as PandasDFrame
 from ..interface.pandas import DataFrameView
@@ -104,8 +104,14 @@ class Comparison(ComparisonInterface):
         cls.equality_type_funcs[np.float32] =   cls.compare_floats
         cls.equality_type_funcs[np.float64] =   cls.compare_floats
 
+        # List and tuple comparisons
+        cls.equality_type_funcs[list] =         cls.compare_lists
+        cls.equality_type_funcs[tuple] =        cls.compare_tuples
+
+
         #Dictionary comparisons
-        cls.equality_type_funcs[dict] =        cls.compare_dictionaries
+        cls.equality_type_funcs[dict] =         cls.compare_dictionaries
+        cls.equality_type_funcs[OrderedDict] =  cls.compare_dictionaries
 
         # Numpy array comparison
         cls.equality_type_funcs[np.ndarray] =   cls.compare_arrays
@@ -206,6 +212,27 @@ class Comparison(ComparisonInterface):
         for k in keys:
             cls.assertEqual(d1[k], d2[k])
 
+
+    @classmethod
+    def compare_lists(cls, l1, l2, msg=None):
+        try:
+            cls.assertEqual(len(l1), len(l2))
+            for v1, v2 in zip(l1, l2):
+                cls.assertEqual(v1, v2)
+        except AssertionError:
+            raise AssertionError(msg or '%s != %s' % (repr(l1), repr(l2)))
+
+
+    @classmethod
+    def compare_tuples(cls, t1, t2, msg=None):
+        try:
+            cls.assertEqual(len(t1), len(t2))
+            for i1, i2 in zip(t1, t2):
+                cls.assertEqual(i1, i2)
+        except AssertionError:
+            raise AssertionError(msg or '%s != %s' % (repr(t1), repr(t2)))
+
+
     #=====================#
     # Literal comparisons #
     #=====================#
@@ -237,23 +264,36 @@ class Comparison(ComparisonInterface):
 
     @classmethod
     def compare_dimensions(cls, dim1, dim2, msg=None):
+
+        # 'Weak' equality semantics
         if dim1.name != dim2.name:
             raise cls.failureException("Dimension names mismatched: %s != %s"
                                        % (dim1.name, dim2.name))
-        if dim1.cyclic != dim2.cyclic:
-            raise cls.failureException("Dimension cyclic declarations mismatched.")
-        if dim1.range != dim2.range:
-            raise cls.failureException("Dimension ranges mismatched: %s != %s"
-                                       % (dim1.range, dim2.range))
-        if dim1.type != dim2.type:
-            raise cls.failureException("Dimension type declarations mismatched: %s != %s"
-                                       % (dim1.type,dim2.type))
-        if dim1.unit != dim2.unit:
-            raise cls.failureException("Dimension unit declarations mismatched: %s != %s"
-                                       % (dim1.unit , dim2.unit))
-        if dim1.values != dim2.values:
-            raise cls.failureException("Dimension value declarations mismatched: %s != %s"
-                                       % (dim1.values , dim2.values))
+        if dim1.label != dim2.label:
+            raise cls.failureException("Dimension labels mismatched: %s != %s"
+                                       % (dim1.label, dim2.label))
+
+        # 'Deep' equality of dimension metadata (all parameters)
+        dim1_params = dict(dim1.get_param_values())
+        dim2_params = dict(dim2.get_param_values())
+
+        # Special handling of deprecated 'initial' values argument
+        dim1_params['values'] = [] if dim1.values=='initial' else dim1.values
+        dim2_params['values'] = [] if dim2.values=='initial' else dim2.values
+
+        if set(dim1_params.keys()) != set(dim2_params.keys()):
+            raise cls.failureException("Dimension parameter sets mismatched: %s != %s"
+                                       % (set(dim1_params.keys()), set(dim2_params.keys())))
+
+        for k in dim1_params.keys():
+            if (dim1.params(k).__class__.__name__ == 'Callable'
+                and dim2.params(k).__class__.__name__ == 'Callable'):
+                continue
+            try:  # This is needed as two lists are not compared by contents using ==
+                cls.assertEqual(dim1_params[k], dim2_params[k], msg=None)
+            except AssertionError as e:
+                msg = 'Dimension parameter %r mismatched: ' % k
+                raise cls.failureException("%s%s" % (msg, str(e)))
 
     @classmethod
     def compare_labelled_data(cls, obj1, obj2, msg=None):
@@ -447,18 +487,18 @@ class Comparison(ComparisonInterface):
     @classmethod
     def compare_dataset(cls, el1, el2, msg='Dataset'):
         cls.compare_dimensioned(el1, el2)
-        if len(el1) != len(el2):
+        if el1.shape[0] != el2.shape[0]:
             raise AssertionError("%s not of matching length." % msg)
         dimension_data = [(d, el1[d], el2[d]) for d in el1.dimensions()]
         for dim, d1, d2 in dimension_data:
             if d1.dtype != d2.dtype:
-                cls.failureException("%s %s columns have different type." % (msg, dim)
+                cls.failureException("%s %s columns have different type." % (msg, dim.pprint_label)
                                      + " First has type %s, and second has type %s."
                                      % (d1, d2))
             if d1.dtype.kind in 'SUOV':
                 if list(d1) == list(d2):
                     cls.failureException("%s along dimension %s not equal." %
-                                         (msg, dim))
+                                         (msg, dim.pprint_label))
             else:
                 cls.compare_arrays(d1, d2, msg)
 
@@ -536,31 +576,27 @@ class Comparison(ComparisonInterface):
 
     @classmethod
     def compare_heatmap(cls, el1, el2, msg='HeatMap'):
-        cls.compare_dimensioned(el1, el2)
-        cls.compare_arrays(el1.data, el2.data, msg)
+        cls.compare_dataset(el1, el2, msg)
 
     @classmethod
     def compare_image(cls, el1, el2, msg='Image'):
-        cls.compare_dimensioned(el1, el2)
-        cls.compare_arrays(el1.data, el2.data, msg)
         cls.bounds_check(el1,el2)
+        cls.compare_dataset(el1, el2, msg)
 
     @classmethod
     def compare_rgb(cls, el1, el2, msg='RGB'):
-        cls.compare_dimensioned(el1, el2)
-        cls.compare_arrays(el1.data, el2.data, msg=msg)
         cls.bounds_check(el1,el2)
+        cls.compare_dataset(el1, el2, msg)
 
     @classmethod
     def compare_hsv(cls, el1, el2, msg='HSV'):
-        cls.compare_dimensioned(el1, el2)
-        cls.compare_arrays(el1.data, el2.data, msg=msg)
         cls.bounds_check(el1,el2)
+        cls.compare_dataset(el1, el2, msg)
 
     @classmethod
     def compare_surface(cls, el1, el2, msg='Surface'):
-        cls.compare_dimensioned(el1, el2)
-        cls.compare_arrays(el1.data, el2.data, msg=msg)
+        cls.bounds_check(el1,el2)
+        cls.compare_dataset(el1, el2, msg)
 
 
     #========#
