@@ -38,7 +38,7 @@ class Path(Element2D):
 
     def __init__(self, data, **params):
         if isinstance(data, tuple):
-            x, y = data
+            x, y = map(np.asarray, data)
             if y.ndim == 1:
                 y = np.atleast_2d(y).T
             if len(x) != y.shape[0]:
@@ -111,7 +111,7 @@ class Contours(Path):
     def dimension_values(self, dim):
         dimension = self.get_dimension(dim, strict=True)
         if dimension in self.vdims:
-            return [self.level]
+            return np.array([self.level])
         return super(Contours, self).dimension_values(dim)
 
 
