@@ -1,17 +1,25 @@
 from collections import defaultdict
 
 import numpy as np
-from bokeh.models import BoxAnnotation
+from bokeh.models import Span
 
-from ...element import HLine, VLine
+from ...element import HLine
 from .element import ElementPlot, text_properties, line_properties
 
 
 class TextPlot(ElementPlot):
 
-    style_opts = text_properties
+    style_opts = text_properties+['color']
     _plot_methods = dict(single='text', batched='text')
 
+    def _glyph_properties(self, plot, element, source, ranges):
+        props = super(TextPlot, self)._glyph_properties(plot, element, source, ranges)
+        props['text_align'] = element.halign
+        props['text_baseline'] = 'middle' if element.valign == 'center' else element.valign
+        if 'color' in props:
+            props['text_color'] = props.pop('color')
+        return props
+
     def get_data(self, element, ranges=None, empty=False):
         mapping = dict(x='x', y='y', text='text')
         if empty:
@@ -20,6 +28,7 @@ class TextPlot(ElementPlot):
             data = dict(x=[element.y], y=[element.x])
         else:
             data = dict(x=[element.x], y=[element.y])
+        self._categorize_data(data, ('x', 'y'), element.dimensions())
         data['text'] = [element.text]
         return (data, mapping)
 
@@ -44,31 +53,22 @@ class LineAnnotationPlot(ElementPlot):
 
     _update_handles = ['glyph']
 
+    _plot_methods = dict(single='Span')
+
     def get_data(self, element, ranges=None, empty=False):
         data, mapping = {}, {}
-        if (isinstance(element, HLine) or
-            (isinstance(element, VLine) and self.invert_axes)):
-            mapping['bottom'] = element.data
-            mapping['top'] = element.data
-        elif (isinstance(element, VLine) or
-              (isinstance(element, HLine) and self.invert_axes)):
-            mapping['left'] = element.data
-            mapping['right'] = element.data
+        mapping['dimension'] = 'width' if isinstance(element, HLine) else 'height'
+        mapping['location'] = element.data
         return (data, mapping)
 
-
     def _init_glyph(self, plot, mapping, properties):
         """
         Returns a Bokeh glyph object.
         """
-        properties.pop('source', None)
-        properties.pop('legend', None)
-        box = BoxAnnotation(plot=plot, level='overlay',
-                            **dict(mapping, **properties))
+        box = Span(level='overlay', **mapping)
         plot.renderers.append(box)
         return None, box
 
-
     def get_extents(self, element, ranges=None):
         return None, None, None, None
 
