@@ -3,22 +3,29 @@ import numpy as np
 
 import param
 
-from bokeh.models import ColumnDataSource, VBox, HBox, GridPlot as BokehGridPlot
+from bokeh.models import (ColumnDataSource, VBox, HBox, Column, Row, Div)
 from bokeh.models.widgets import Panel, Tabs
 
 from ...core import (OrderedDict, CompositeOverlay, Store, Layout, GridMatrix,
-                     AdjointLayout, NdLayout, Empty, GridSpace, HoloMap)
-from ...core import traversal
-from ...core.options import Compositor, SkipRendering
-from ...core.util import basestring, wrap_tuple
+                     AdjointLayout, NdLayout, Empty, GridSpace, HoloMap, Element)
+from ...core.options import Compositor
+from ...core.util import basestring, wrap_tuple, unique_iterator
 from ...element import Histogram
-from ..plot import DimensionedPlot, GenericCompositePlot, GenericLayoutPlot
-from ..util import get_dynamic_mode, initialize_sampled
-from .renderer import BokehRenderer
-from .util import bokeh_version, layout_padding, pad_plots
+from ..plot import (DimensionedPlot, GenericCompositePlot, GenericLayoutPlot,
+                    GenericElementPlot)
+from ..util import attach_streams
+from .util import (bokeh_version, layout_padding, pad_plots,
+                   filter_toolboxes, make_axis, update_shared_sources,
+                   empty_plot)
 
 if bokeh_version >= '0.12':
     from bokeh.layouts import gridplot
+else:
+    from bokeh.models import GridPlot as BokehGridPlot
+from bokeh.plotting.helpers import _known_tools as known_tools
+
+TOOLS = {name: tool if isinstance(tool, basestring) else type(tool())
+         for name, tool in known_tools.items()}
 
 
 class BokehPlot(DimensionedPlot):
@@ -42,12 +49,13 @@ class BokehPlot(DimensionedPlot):
         The formatting string for the title of this plot, allows defining
         a label group separator and dimension labels.""")
 
-    renderer = BokehRenderer
+    backend = 'bokeh'
 
     @property
     def document(self):
         return self._document
 
+
     @document.setter
     def document(self, doc):
         self._document = doc
@@ -55,11 +63,13 @@ class BokehPlot(DimensionedPlot):
             for plot in self.subplots.values():
                 plot.document = doc
 
+
     def __init__(self, *args, **params):
         super(BokehPlot, self).__init__(*args, **params)
         self._document = None
         self.root = None
 
+
     def get_data(self, element, ranges=None, empty=False):
         """
         Returns the data from an element in the appropriate format for
@@ -70,12 +80,16 @@ class BokehPlot(DimensionedPlot):
         raise NotImplementedError
 
 
-    def set_document(self, document):
+    def push(self):
         """
-        Sets the current document on all subplots.
+        Pushes updated plot data via the Comm.
         """
-        for plot in self.traverse(lambda x: x):
-            plot.document = document
+        if self.renderer.mode == 'server':
+            return
+        if self.comm is None:
+            raise Exception('Renderer does not have a comm.')
+        diff = self.renderer.diff(self)
+        self.comm.send(diff)
 
 
     def set_root(self, root):
@@ -97,8 +111,7 @@ class BokehPlot(DimensionedPlot):
         """
         Update datasource with data for a new frame.
         """
-        for k, v in data.items():
-            source.data[k] = v
+        source.data.update(data)
 
     @property
     def state(self):
@@ -139,6 +152,8 @@ class BokehPlot(DimensionedPlot):
                                and 'source' in x.handles)
         data_sources = self.traverse(get_sources, [filter_fn])
         grouped_sources = groupby(sorted(data_sources, key=lambda x: x[0]), lambda x: x[0])
+        shared_sources = []
+        source_cols = {}
         for _, group in grouped_sources:
             group = list(group)
             if len(group) > 1:
@@ -155,37 +170,132 @@ class BokehPlot(DimensionedPlot):
                     else:
                         renderer.update(source=new_source)
                     plot.handles['source'] = new_source
+                shared_sources.append(new_source)
+                source_cols[id(new_source)] = [c for c in new_source.data]
+        self.handles['shared_sources'] = shared_sources
+        self.handles['source_cols'] = source_cols
 
 
 
-class GridPlot(BokehPlot, GenericCompositePlot):
+class CompositePlot(BokehPlot):
+    """
+    CompositePlot is an abstract baseclass for plot types that draw
+    render multiple axes. It implements methods to add an overall title
+    to such a plot.
+    """
+
+    fontsize = param.Parameter(default={'title': '16pt'}, allow_None=True,  doc="""
+       Specifies various fontsizes of the displayed text.
+
+       Finer control is available by supplying a dictionary where any
+       unmentioned keys reverts to the default sizes, e.g:
+
+          {'title': '15pt'}""")
+
+    _title_template = "<span style='font-size: {fontsize}'><b>{title}</b></font>"
+
+    _merged_tools = ['pan', 'box_zoom', 'box_select', 'lasso_select',
+                     'poly_select', 'ypan', 'xpan']
+
+    def _update_callbacks(self, plot):
+        """
+        Iterates over all subplots and updates existing CustomJS
+        callbacks with models that were replaced when compositing subplots
+        into a CompositePlot
+        """
+        subplots = self.traverse(lambda x: x, [GenericElementPlot])
+        merged_tools = {t: list(plot.select({'type': TOOLS[t]}))
+                        for t in self._merged_tools}
+        for subplot in subplots:
+            for cb in subplot.callbacks:
+                for c in cb.callbacks:
+                    for tool, objs in merged_tools.items():
+                        if tool in c.args and objs:
+                            c.args[tool] = objs[0]
+
+
+    def _get_title(self, key):
+        title_div = None
+        title = self._format_title(key) if self.show_title else ''
+        if title:
+            fontsize = self._fontsize('title')
+            title_tags = self._title_template.format(title=title,
+                                                     **fontsize)
+            if 'title' in self.handles:
+                title_div = self.handles['title']
+            else:
+                title_div = Div()
+            title_div.text = title_tags
+        return title_div
+
+    @property
+    def current_handles(self):
+        """
+        Should return a list of plot objects that have changed and
+        should be updated.
+        """
+        return [self.handles['title']] if 'title' in self.handles else []
+
+
+
+class GridPlot(CompositePlot, GenericCompositePlot):
     """
     Plot a group of elements in a grid layout based on a GridSpace element
     object.
     """
 
-    def __init__(self, layout, ranges=None, keys=None, dimensions=None,
-                 layout_num=1, **params):
+    axis_offset = param.Integer(default=50, doc="""
+        Number of pixels to adjust row and column widths and height by
+        to compensate for shared axes.""")
+
+    fontsize = param.Parameter(default={'title': '16pt'},
+                               allow_None=True,  doc="""
+       Specifies various fontsizes of the displayed text.
+
+       Finer control is available by supplying a dictionary where any
+       unmentioned keys reverts to the default sizes, e.g:
+
+          {'title': '15pt'}""")
+
+    shared_xaxis = param.Boolean(default=False, doc="""
+        If enabled the x-axes of the GridSpace will be drawn from the
+        objects inside the Grid rather than the GridSpace dimensions.""")
+
+    shared_yaxis = param.Boolean(default=False, doc="""
+        If enabled the x-axes of the GridSpace will be drawn from the
+        objects inside the Grid rather than the GridSpace dimensions.""")
+
+    xaxis = param.ObjectSelector(default=True,
+                                 objects=['bottom', 'top', None, True, False], doc="""
+        Whether and where to display the xaxis, supported options are
+        'bottom', 'top' and None.""")
+
+    yaxis = param.ObjectSelector(default=True,
+                                 objects=['left', 'right', None, True, False], doc="""
+        Whether and where to display the yaxis, supported options are
+        'left', 'right' and None.""")
+
+    xrotation = param.Integer(default=0, bounds=(0, 360), doc="""
+        Rotation angle of the xticks.""")
+
+    yrotation = param.Integer(default=0, bounds=(0, 360), doc="""
+        Rotation angle of the yticks.""")
+
+    plot_size = param.Integer(default=120, doc="""
+        Defines the width and height of each plot in the grid""")
+
+    def __init__(self, layout, ranges=None, layout_num=1, keys=None, **params):
         if not isinstance(layout, GridSpace):
             raise Exception("GridPlot only accepts GridSpace.")
-
-        self.layout = layout
-        self.rows, self.cols = layout.shape
-        self.layout_num = layout_num
-        extra_opts = self.lookup_options(layout, 'plot').options
-        if not keys or not dimensions:
-            dimensions, keys = traversal.unique_dimkeys(layout)
-        if 'uniform' not in params:
-            params['uniform'] = traversal.uniform(layout)
-
-        dynamic, sampled = get_dynamic_mode(layout)
-        if sampled:
-            initialize_sampled(layout, dimensions, keys[0])
-
-        super(GridPlot, self).__init__(keys=keys, dimensions=dimensions,
-                                       dynamic=dynamic,
-                                       **dict(extra_opts, **params))
+        super(GridPlot, self).__init__(layout=layout, layout_num=layout_num,
+                                       ranges=ranges, keys=keys, **params)
+        self.cols, self.rows = layout.shape
         self.subplots, self.layout = self._create_subplots(layout, ranges)
+        if self.top_level:
+            self.comm = self.init_comm()
+            self.traverse(lambda x: setattr(x, 'comm', self.comm))
+            self.traverse(lambda x: attach_streams(self, x.hmap, 2),
+                          [GenericElementPlot])
 
 
     def _create_subplots(self, layout, ranges):
@@ -198,8 +308,8 @@ class GridPlot(BokehPlot, GenericCompositePlot):
                                     for key in self.keys])
         collapsed_layout = layout.clone(shared_data=False, id=layout.id)
         for i, coord in enumerate(layout.keys(full_grid=True)):
-            r = i % self.cols
-            c = i // self.cols
+            r = i % self.rows
+            c = i // self.rows
 
             if not isinstance(coord, tuple): coord = (coord,)
             view = layout.data.get(coord, None)
@@ -211,26 +321,35 @@ class GridPlot(BokehPlot, GenericCompositePlot):
                 vtype = None
 
             # Create axes
+            offset = self.axis_offset
             kwargs = {}
             if c == 0 and r != 0:
-                kwargs['xaxis'] = 'bottom-bare'
-                kwargs['width'] = 150
+                kwargs['xaxis'] = None
+                kwargs['width'] = self.plot_size+offset
             if c != 0 and r == 0:
-                kwargs['yaxis'] = 'left-bare'
-                kwargs['height'] = 150
+                kwargs['yaxis'] = None
+                kwargs['height'] = self.plot_size+offset
             if c == 0 and r == 0:
-                kwargs['width'] = 150
-                kwargs['height'] = 150
+                kwargs['width'] = self.plot_size+offset
+                kwargs['height'] = self.plot_size+offset
             if r != 0 and c != 0:
-                kwargs['xaxis'] = 'bottom-bare'
-                kwargs['yaxis'] = 'left-bare'
+                kwargs['xaxis'] = None
+                kwargs['yaxis'] = None
 
-            if 'width' not in kwargs:
-                kwargs['width'] = 105
-            if 'height' not in kwargs:
-                kwargs['height'] = 105
+            if 'width' not in kwargs or not self.shared_yaxis:
+                kwargs['width'] = self.plot_size
+            if 'height' not in kwargs or not self.shared_xaxis:
+                kwargs['height'] = self.plot_size
             if 'border' not in kwargs:
-                kwargs['border'] = 0
+                kwargs['border'] = 3
+
+            kwargs['show_legend'] = False
+
+            if not self.shared_xaxis:
+                kwargs['xaxis'] = None
+
+            if not self.shared_yaxis:
+                kwargs['yaxis'] = None
 
             if isinstance(layout, GridMatrix):
                 if view.traverse(lambda x: x, [Histogram]):
@@ -245,40 +364,45 @@ class GridPlot(BokehPlot, GenericCompositePlot):
             else:
                 subplot = plotting_class(view, dimensions=self.dimensions,
                                          show_title=False, subplot=True,
+                                         renderer=self.renderer,
                                          ranges=frame_ranges, uniform=self.uniform,
                                          keys=self.keys, **dict(opts, **kwargs))
                 collapsed_layout[coord] = (subplot.layout
                                            if isinstance(subplot, GenericCompositePlot)
                                            else subplot.hmap)
                 subplots[coord] = subplot
-        self.sync_tools(subplots)
         return subplots, collapsed_layout
 
 
-    def sync_tools(self, subplots):
-        tools = list({t for p in subplots.values() for t in p.tools})
-        for plot in subplots.values():
-            plot.tools = tools
-
-
     def initialize_plot(self, ranges=None, plots=[]):
         ranges = self.compute_ranges(self.layout, self.keys[-1], None)
-        plots = [[] for r in range(self.cols)]
         passed_plots = list(plots)
+        plots = [[None for c in range(self.cols)] for r in range(self.rows)]
         for i, coord in enumerate(self.layout.keys(full_grid=True)):
-            r = i % self.cols
+            r = i % self.rows
+            c = i // self.rows
             subplot = self.subplots.get(wrap_tuple(coord), None)
             if subplot is not None:
                 plot = subplot.initialize_plot(ranges=ranges, plots=passed_plots)
-                plots[r].append(plot)
-                passed_plots.append(plots[r][-1])
+                plots[r][c] = plot
+                passed_plots.append(plot)
             else:
-                plots[r].append(None)
                 passed_plots.append(None)
+
         if bokeh_version < '0.12':
-            self.handles['plot'] = BokehGridPlot(children=plots[::-1])
+            plot = BokehGridPlot(children=plots[::-1])
         else:
-            self.handles['plot'] = gridplot(plots[::-1])
+            plot = gridplot(plots[::-1])
+
+        plot = self._make_axes(plot)
+
+        title = self._get_title(self.keys[-1])
+        if title:
+            plot = Column(title, plot)
+            self.handles['title'] = title
+
+        self._update_callbacks(plot)
+        self.handles['plot'] = plot
         self.handles['plots'] = plots
         if self.shared_datasource:
             self.sync_sources()
@@ -287,6 +411,47 @@ class GridPlot(BokehPlot, GenericCompositePlot):
         return self.handles['plot']
 
 
+    def _make_axes(self, plot):
+        width, height = self.renderer.get_size(plot)
+        x_axis, y_axis = None, None
+        if self.xaxis:
+            flip = self.shared_xaxis
+            rotation = self.xrotation
+            lsize = self._fontsize('xlabel').get('fontsize')
+            tsize = self._fontsize('xticks', common=False).get('fontsize')
+            xfactors = list(unique_iterator(self.layout.dimension_values(0)))
+            x_axis = make_axis('x', width, xfactors, self.layout.kdims[0],
+                               flip=flip, rotation=rotation, label_size=lsize,
+                               tick_size=tsize)
+        if self.yaxis and self.layout.ndims > 1:
+            flip = self.shared_yaxis
+            rotation = self.yrotation
+            lsize = self._fontsize('ylabel').get('fontsize')
+            tsize = self._fontsize('yticks', common=False).get('fontsize')
+            yfactors = list(unique_iterator(self.layout.dimension_values(1)))
+            y_axis = make_axis('y', height, yfactors, self.layout.kdims[1],
+                               flip=flip, rotation=rotation, label_size=lsize,
+                               tick_size=tsize)
+        if x_axis and y_axis:
+            plot = filter_toolboxes(plot)
+            r1, r2 = ([y_axis, plot], [None, x_axis])
+            if self.shared_xaxis:
+                r1, r2 = r2, r1
+            if self.shared_yaxis:
+                r1, r2 = r1[::-1], r2[::-1]
+            models = layout_padding([r1, r2], self.renderer)
+            plot = gridplot(models)
+        elif y_axis:
+            models = [y_axis, plot]
+            if self.shared_yaxis: models = models[::-1]
+            plot = Row(*models)
+        elif x_axis:
+            models = [plot, x_axis]
+            if self.shared_xaxis: models = models[::-1]
+            plot = Column(*models)
+        return plot
+
+    @update_shared_sources
     def update_frame(self, key, ranges=None):
         """
         Update the internal state of the Plot to represent the given
@@ -295,13 +460,16 @@ class GridPlot(BokehPlot, GenericCompositePlot):
         """
         ranges = self.compute_ranges(self.layout, key, ranges)
         for coord in self.layout.keys(full_grid=True):
-            subplot = self.subplots.get(coord, None)
+            subplot = self.subplots.get(wrap_tuple(coord), None)
             if subplot is not None:
                 subplot.update_frame(key, ranges)
+        title = self._get_title(key)
+        if title:
+            self.handles['title']
 
 
 
-class LayoutPlot(BokehPlot, GenericLayoutPlot):
+class LayoutPlot(CompositePlot, GenericLayoutPlot):
 
     shared_axes = param.Boolean(default=True, doc="""
         Whether axes should be shared across plots""")
@@ -314,10 +482,14 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
     tabs = param.Boolean(default=False, doc="""
         Whether to display overlaid plots in separate panes""")
 
-    def __init__(self, layout, **params):
-        super(LayoutPlot, self).__init__(layout, **params)
+    def __init__(self, layout, keys=None, **params):
+        super(LayoutPlot, self).__init__(layout, keys=keys, **params)
         self.layout, self.subplots, self.paths = self._init_layout(layout)
-
+        if self.top_level:
+            self.comm = self.init_comm()
+            self.traverse(lambda x: setattr(x, 'comm', self.comm))
+            self.traverse(lambda x: attach_streams(self, x.hmap, 2),
+                          [GenericElementPlot])
 
     def _init_layout(self, layout):
         # Situate all the Layouts in the grid and compute the gridspec
@@ -332,7 +504,7 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
         layout_subplots, layouts, paths = {}, {}, {}
         for r, c in self.coords:
             # Get view at layout position and wrap in AdjointLayout
-            key, view = layout_items.get((r, c), (None, None))
+            key, view = layout_items.get((c, r) if self.transpose else (r, c), (None, None))
             view = view if isinstance(view, AdjointLayout) else AdjointLayout([view])
             layouts[(r, c)] = view
             paths[r, c] = key
@@ -355,7 +527,10 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
             obj = layouts[(r, c)]
             empty = isinstance(obj.main, Empty)
             if empty:
-                obj = AdjointLayout([])
+                continue
+            elif view is None or not view.traverse(lambda x: x, [Element]):
+                self.warning('%s is empty, skipping subplot.' % obj.main)
+                continue
             else:
                 layout_count += 1
             subplot_data = self._create_subplots(obj, positions,
@@ -383,17 +558,15 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
         """
         subplots = {}
         adjoint_clone = layout.clone(shared_data=False, id=layout.id)
-        subplot_opts = dict(adjoined=layout)
         main_plot = None
         for pos in positions:
             # Pos will be one of 'main', 'top' or 'right' or None
             element = layout.get(pos, None)
-            if element is None:
+            if element is None or not element.traverse(lambda x: x, [Element]):
                 continue
 
+            subplot_opts = dict(adjoined=main_plot)
             # Options common for any subplot
-            if type(element) in (NdLayout, Layout):
-                raise SkipRendering("Cannot plot nested Layouts.")
             vtype = element.type if isinstance(element, HoloMap) else element.__class__
             plot_type = Store.registry[self.renderer.backend].get(vtype, None)
             plotopts = self.lookup_options(element, 'plot').options
@@ -404,11 +577,11 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
                     yaxis = 'right-bare' if 'bare' in plot_type.yaxis else 'right'
                     side_opts = dict(height=main_plot.height, yaxis=yaxis,
                                      width=plot_type.width, invert_axes=True,
-                                     show_labels=['y'], xticks=1, xaxis=main_plot.xaxis)
+                                     labelled=['y'], xticks=1, xaxis=main_plot.xaxis)
                 else:
                     xaxis = 'top-bare' if 'bare' in plot_type.xaxis else 'top'
                     side_opts = dict(width=main_plot.width, xaxis=xaxis,
-                                     height=plot_type.height, show_labels=['x'],
+                                     height=plot_type.height, labelled=['x'],
                                      yticks=1, yaxis=main_plot.yaxis)
 
             # Override the plotopts as required
@@ -420,15 +593,15 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
                 self.warning("Bokeh plotting class for %s type not found, object will "
                              "not be rendered." % vtype.__name__)
                 continue
-            if plot_type in [GridPlot, LayoutPlot]:
-                self.tabs = True
             num = num if len(self.coords) > 1 else 0
             subplot = plot_type(element, keys=self.keys,
                                 dimensions=self.dimensions,
                                 layout_dimensions=layout_dimensions,
                                 ranges=ranges, subplot=True,
                                 uniform=self.uniform, layout_num=num,
-                                **plotopts)
+                                renderer=self.renderer,
+                                **dict({'shared_axes': self.shared_axes},
+                                       **plotopts))
             subplots[pos] = subplot
             if isinstance(plot_type, type) and issubclass(plot_type, GenericCompositePlot):
                 adjoint_clone[pos] = subplots[pos].layout
@@ -440,10 +613,10 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
         return subplots, adjoint_clone
 
 
-    def initialize_plot(self, ranges=None):
+    def initialize_plot(self, plots=None, ranges=None):
         ranges = self.compute_ranges(self.layout, self.keys[-1], None)
+        passed_plots = [] if plots is None else plots
         plots = [[] for _ in range(self.rows)]
-        passed_plots = []
         tab_titles = {}
         insert_rows, insert_cols = [], []
         adjoined = False
@@ -489,6 +662,8 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
                 if len(subplots) == 1 and c in insert_cols:
                     plots[r+offset].append(None)
                 passed_plots.append(subplots[0])
+            else:
+                plots[r+offset] += [empty_plot(0, 0)]
 
             if self.tabs:
                 if isinstance(self.layout, Layout):
@@ -500,22 +675,17 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
 
         # Replace None types with empty plots
         # to avoid bokeh bug
-        if adjoined:
-            plots = layout_padding(plots)
+        plots = layout_padding(plots, self.renderer)
 
-        # Determine the most appropriate composite plot type
-        # If the object cannot be displayed in a single layout
-        # it will be split into Tabs, for 1-row or 1-column
-        # Layouts we use the vplot and hplots.
-        # If there is a table and multiple rows and columns
-        # everything will be forced to a vertical layout
+        # Wrap in appropriate layout model
         if self.tabs:
-            panels = [Panel(child=child, title=str(tab_titles.get(r, c)))
+            panels = [Panel(child=child, title=str(tab_titles.get((r, c))))
                       for r, row in enumerate(plots)
                       for c, child in enumerate(row)
                       if child is not None]
             layout_plot = Tabs(tabs=panels)
         elif bokeh_version >= '0.12':
+            plots = filter_toolboxes(plots)
             plots, width = pad_plots(plots)
             layout_plot = gridplot(children=plots, width=width)
         elif len(plots) == 1 and not adjoined:
@@ -525,6 +695,12 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
         else:
             layout_plot = BokehGridPlot(children=plots)
 
+        title = self._get_title(self.keys[-1])
+        if title:
+            self.handles['title'] = title
+            layout_plot = Column(title, layout_plot)
+
+        self._update_callbacks(layout_plot)
         self.handles['plot'] = layout_plot
         self.handles['plots'] = plots
         if self.shared_datasource:
@@ -534,7 +710,7 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
 
         return self.handles['plot']
 
-
+    @update_shared_sources
     def update_frame(self, key, ranges=None):
         """
         Update the internal state of the Plot to represent the given
@@ -546,9 +722,13 @@ class LayoutPlot(BokehPlot, GenericLayoutPlot):
             subplot = self.subplots.get((r, c), None)
             if subplot is not None:
                 subplot.update_frame(key, ranges)
+        title = self._get_title(key)
+        if title:
+            self.handles['title'] = title
 
 
-class AdjointLayoutPlot(BokehPlot, GenericCompositePlot):
+
+class AdjointLayoutPlot(BokehPlot):
 
     layout_dict = {'Single': {'positions': ['main']},
                    'Dual':   {'positions': ['main', 'right']},
