@@ -1,10 +1,11 @@
-import math
+import math, copy
 
 import param
 import numpy as np
+import matplotlib as mpl
 import matplotlib.pyplot as plt
+import matplotlib.colors as mpl_colors
 from matplotlib import ticker
-from matplotlib import colors
 from matplotlib.dates import date2num
 
 from ...core import util
@@ -13,8 +14,9 @@ from ...core import (OrderedDict, NdOverlay, DynamicMap,
 from ...core.options import abbreviated_exception
 from ..plot import GenericElementPlot, GenericOverlayPlot
 from ..util import dynamic_update
-from .plot import MPLPlot
+from .plot import MPLPlot, mpl_rc_context
 from .util import wrap_formatter
+from distutils.version import LooseVersion
 
 
 class ElementPlot(GenericElementPlot, MPLPlot):
@@ -130,7 +132,7 @@ class ElementPlot(GenericElementPlot, MPLPlot):
                 self.warning("Plotting hook %r could not be applied:\n\n %s" % (hook, e))
 
 
-    def _finalize_axis(self, key, title=None, dimensions=None, ranges=None, xticks=None,
+    def _finalize_axis(self, key, element=None, title=None, dimensions=None, ranges=None, xticks=None,
                        yticks=None, zticks=None, xlabel=None, ylabel=None, zlabel=None):
         """
         Applies all the axis settings before the axis or figure is returned.
@@ -139,7 +141,8 @@ class ElementPlot(GenericElementPlot, MPLPlot):
         When the number of the frame is supplied as n, this method looks
         up and computes the appropriate title, axis labels and axis bounds.
         """
-        element = self._get_frame(key)
+        if element is None:
+            element = self._get_frame(key)
         self.current_frame = element
         if not dimensions and element and not self.subplots:
             el = element.traverse(lambda x: x, [Element])
@@ -149,13 +152,19 @@ class ElementPlot(GenericElementPlot, MPLPlot):
         subplots = list(self.subplots.values()) if self.subplots else []
         if self.zorder == 0 and key is not None:
             if self.bgcolor:
-                axis.set_axis_bgcolor(self.bgcolor)
-
+                if LooseVersion(mpl.__version__) <= '1.5.9':
+                    axis.set_axis_bgcolor(self.bgcolor)
+                else:
+                    axis.set_facecolor(self.bgcolor)
+              
             # Apply title
-            title = None if self.zorder > 0 else self._format_title(key)
+            title = self._format_title(key)
             if self.show_title and title is not None:
                 fontsize = self._fontsize('title')
-                self.handles['title'] = axis.set_title(title, **fontsize)
+                if 'title' in self.handles:
+                    self.handles['title'].set_text(title)
+                else:
+                    self.handles['title'] = axis.set_title(title, **fontsize)
 
             # Apply subplot label
             self._subplot_label(axis)
@@ -166,12 +175,11 @@ class ElementPlot(GenericElementPlot, MPLPlot):
                 if dimensions:
                     self._set_labels(axis, dimensions, xlabel, ylabel, zlabel)
 
-                # Set axes limits
-                self._set_axis_limits(axis, element, subplots, ranges)
-
                 if not subplots:
                     legend = axis.get_legend()
-                    if legend: legend.set_visible(self.show_legend)
+                    if legend:
+                        legend.set_visible(self.show_legend)
+                        self.handles["bbox_extra_artists"] += [legend]
                     axis.xaxis.grid(self.show_grid)
                     axis.yaxis.grid(self.show_grid)
 
@@ -189,8 +197,11 @@ class ElementPlot(GenericElementPlot, MPLPlot):
                 if self.apply_ticks:
                     self._finalize_ticks(axis, dimensions, xticks, yticks, zticks)
 
+                # Set axes limits
+                self._set_axis_limits(axis, element, subplots, ranges)
+
             # Apply aspects
-            if not (self.logx or self.logy):
+            if self.aspect is not None and self.projection != 'polar':
                 self._set_aspect(axis, self.aspect)
 
         if not subplots and not self.drawn:
@@ -238,8 +249,9 @@ class ElementPlot(GenericElementPlot, MPLPlot):
             self._set_axis_ticks(axis.zaxis, zticks, log=self.logz,
                                  rotation=self.zrotation)
 
-        tick_fontsize = self._fontsize('ticks','labelsize',common=False)
-        if tick_fontsize: axis.tick_params(**tick_fontsize)
+        for ax, ax_obj in zip('xy', [axis.xaxis, axis.yaxis]):
+            tick_fontsize = self._fontsize('%sticks' % ax,'labelsize',common=False)
+            if tick_fontsize: ax_obj.set_tick_params(**tick_fontsize)
 
 
     def _finalize_artist(self, element):
@@ -285,13 +297,19 @@ class ElementPlot(GenericElementPlot, MPLPlot):
         """
         Set the aspect on the axes based on the aspect setting.
         """
-        if aspect and aspect == 'square':
-            axes.set_aspect((1./axes.get_data_ratio()))
-        elif aspect not in [None, 'square']:
-            if isinstance(aspect, util.basestring):
-                axes.set_aspect(aspect)
-            else:
-                axes.set_aspect(((1./axes.get_data_ratio()))/aspect)
+        if isinstance(aspect, util.basestring) and aspect != 'square':
+            axes.set_aspect(aspect)
+            return
+
+        (x0, x1), (y0, y1) = axes.get_xlim(), axes.get_ylim()
+        xsize = np.log(x1) - np.log(x0) if self.logx else x1-x0
+        ysize = np.log(y1) - np.log(y0) if self.logy else y1-y0
+        xsize = max(abs(xsize), 1e-30)
+        ysize = max(abs(ysize), 1e-30)
+        data_ratio = 1./(ysize/xsize)
+        if aspect != 'square':
+            data_ratio = data_ratio/aspect
+        axes.set_aspect(float(data_ratio))
 
 
     def _set_axis_limits(self, axis, view, subplots, ranges):
@@ -406,6 +424,7 @@ class ElementPlot(GenericElementPlot, MPLPlot):
             tick.set_rotation(rotation)
 
 
+    @mpl_rc_context
     def update_frame(self, key, ranges=None, element=None):
         """
         Set the plot(s) to the given frame number.  Operates by
@@ -418,7 +437,7 @@ class ElementPlot(GenericElementPlot, MPLPlot):
         reused = isinstance(self.hmap, DynamicMap) and self.overlaid
         if not reused and element is None:
             element = self._get_frame(key)
-        else:
+        elif element is not None:
             self.current_key = key
             self.current_frame = element
 
@@ -445,9 +464,11 @@ class ElementPlot(GenericElementPlot, MPLPlot):
         label = element.label if self.show_legend else ''
         style = dict(label=label, zorder=self.zorder, **self.style[self.cyclic_index])
         axis_kwargs = self.update_handles(key, axis, element, ranges, style)
-        self._finalize_axis(key, ranges=ranges, **(axis_kwargs if axis_kwargs else {}))
+        self._finalize_axis(key, element=element, ranges=ranges,
+                            **(axis_kwargs if axis_kwargs else {}))
 
 
+    @mpl_rc_context
     def initialize_plot(self, ranges=None):
         element = self.hmap.last
         ax = self.handles['axis']
@@ -468,7 +489,8 @@ class ElementPlot(GenericElementPlot, MPLPlot):
             handles = self.init_artists(ax, plot_data, plot_kwargs)
         self.handles.update(handles)
 
-        return self._finalize_axis(self.keys[-1], ranges=ranges, **axis_kwargs)
+        return self._finalize_axis(self.keys[-1], element=element, ranges=ranges,
+                                   **axis_kwargs)
 
 
     def init_artists(self, ax, plot_args, plot_kwargs):
@@ -512,8 +534,13 @@ class ColorbarPlot(ElementPlot):
     colorbar = param.Boolean(default=False, doc="""
         Whether to draw a colorbar.""")
 
-    cbar_width = param.Number(default=0.05, doc="""
-        Width of the colorbar as a fraction of the main plot""")
+    clipping_colors = param.Dict(default={}, doc="""
+        Dictionary to specify colors for clipped values, allows
+        setting color for NaN values and for values above and below
+        the min and max value. The min, max or NaN color may specify
+        an RGB(A) color as a color hex string of the form #FFFFFF or
+        #FFFFFFFF or a length 3 or length 4 tuple specifying values in
+        the range 0-1 or a named HTML color.""")
 
     cbar_padding = param.Number(default=0.01, doc="""
         Padding between colorbar and other plots.""")
@@ -525,23 +552,30 @@ class ColorbarPlot(ElementPlot):
         set to None default matplotlib ticking behavior is
         applied.""")
 
+    cbar_width = param.Number(default=0.05, doc="""
+        Width of the colorbar as a fraction of the main plot""")
+
     symmetric = param.Boolean(default=False, doc="""
         Whether to make the colormap symmetric around zero.""")
 
     _colorbars = {}
 
+    def __init__(self, *args, **kwargs):
+        super(ColorbarPlot, self).__init__(*args, **kwargs)
+        self._cbar_extend = 'neither'
+
     def _adjust_cbar(self, cbar, label, dim):
         noalpha = math.floor(self.style[self.cyclic_index].get('alpha', 1)) == 1
         if (cbar.solids and noalpha):
             cbar.solids.set_edgecolor("face")
         cbar.set_label(label)
         if isinstance(self.cbar_ticks, ticker.Locator):
-            cbar.set_major_locator(self.cbar_ticks)
+            cbar.ax.yaxis.set_major_locator(self.cbar_ticks)
         elif self.cbar_ticks == 0:
             cbar.set_ticks([])
         elif isinstance(self.cbar_ticks, int):
             locator = ticker.MaxNLocator(self.cbar_ticks)
-            cbar.set_major_locator(locator)
+            cbar.ax.yaxis.set_major_locator(locator)
         elif isinstance(self.cbar_ticks, list):
             if all(isinstance(t, tuple) for t in self.cbar_ticks):
                 ticks, labels = zip(*self.cbar_ticks)
@@ -577,9 +611,12 @@ class ColorbarPlot(ElementPlot):
 
         # Get colorbar label
         dim = element.get_dimension(dim)
-        if dim is None:
-            dim = element.vdims[0]
-        label = str(dim)
+        if dim:
+            label = dim.pprint_label
+        elif element.vdims:
+            label = element.vdims[0].pprint_label
+        elif dim is None:
+            label = ''
 
         padding = self.cbar_padding
         width = self.cbar_width
@@ -588,7 +625,7 @@ class ColorbarPlot(ElementPlot):
             scaled_w = w*width
             cax = fig.add_axes([l+w+padding+(scaled_w+padding+w*0.15)*offset,
                                 b, scaled_w, h])
-            cbar = plt.colorbar(artist, cax=cax)
+            cbar = fig.colorbar(artist, cax=cax, ax=axis, extend=self._cbar_extend)
             self._adjust_cbar(cbar, label, dim)
             self.handles['cax'] = cax
             self.handles['cbar'] = cbar
@@ -604,7 +641,6 @@ class ColorbarPlot(ElementPlot):
         ColorbarPlot._colorbars[id(axis)] = (ax_colorbars, (l, b, w, h))
 
 
-
     def _norm_kwargs(self, element, ranges, opts, vdim):
         """
         Returns valid color normalization kwargs
@@ -612,19 +648,60 @@ class ColorbarPlot(ElementPlot):
         """
         clim = opts.pop('clims', None)
         if clim is None:
-            clim = ranges[vdim.name] if vdim.name in ranges else element.range(vdim)
-            if self.symmetric:
-                clim = -np.abs(clim).max(), np.abs(clim).max()
+            cs = element.dimension_values(vdim)
+            if not isinstance(cs, np.ndarray):
+                cs = np.array(cs)
+            if len(cs) and cs.dtype.kind in 'if':
+                clim = ranges[vdim.name] if vdim.name in ranges else element.range(vdim)
+                if self.symmetric:
+                    clim = -np.abs(clim).max(), np.abs(clim).max()
+            else:
+                clim = (0, len(np.unique(cs)))
         if self.logz:
             if self.symmetric:
-                norm = colors.SymLogNorm(vmin=clim[0], vmax=clim[1],
+                norm = mpl_colors.SymLogNorm(vmin=clim[0], vmax=clim[1],
                                          linthresh=clim[1]/np.e)
             else:
-                norm = colors.LogNorm(vmin=clim[0], vmax=clim[1])
+                norm = mpl_colors.LogNorm(vmin=clim[0], vmax=clim[1])
             opts['norm'] = norm
         opts['vmin'] = clim[0]
         opts['vmax'] = clim[1]
 
+        # Check whether the colorbar should indicate clipping
+        values = np.asarray(element.dimension_values(vdim))
+        if values.dtype.kind not in 'OSUM':
+            el_min, el_max = np.nanmin(values), np.nanmax(values)
+        else:
+            el_min, el_max = -np.inf, np.inf
+        vmin = -np.inf if opts['vmin'] is None else opts['vmin']
+        vmax = np.inf if opts['vmax'] is None else opts['vmax']
+        if el_min < vmin and el_max > vmax:
+            self._cbar_extend = 'both'
+        elif el_min < vmin:
+            self._cbar_extend = 'min'
+        elif el_max > vmax:
+            self._cbar_extend = 'max'
+
+        # Define special out-of-range colors on colormap
+        cmap = copy.copy(plt.cm.get_cmap(opts.get('cmap')))
+        colors = {}
+        for k, val in self.clipping_colors.items():
+            if isinstance(val, tuple):
+                colors[k] = {'color': val[:3],
+                             'alpha': val[3] if len(val) > 3 else 1}
+            elif isinstance(val, util.basestring):
+                color = val
+                alpha = 1
+                if color.startswith('#') and len(color) == 9:
+                    alpha = int(color[-2:], 16)/255.
+                    color = color[:-2]
+                colors[k] = {'color': color, 'alpha': alpha}
+        if 'max' in colors: cmap.set_over(**colors['max'])
+        if 'min' in colors: cmap.set_under(**colors['min'])
+        if 'NaN' in colors: cmap.set_bad(**colors['NaN'])
+        opts['cmap'] = cmap
+
+
 
 class LegendPlot(ElementPlot):
 
@@ -671,6 +748,13 @@ class OverlayPlot(LegendPlot, GenericOverlayPlot):
 
     _passed_handles = ['fig', 'axis']
 
+    _propagate_options = ['aspect', 'fig_size', 'xaxis', 'yaxis', 'zaxis',
+                          'labelled', 'bgcolor', 'fontsize', 'invert_axes',
+                          'show_frame', 'show_grid', 'logx', 'logy', 'logz',
+                          'xticks', 'yticks', 'zticks', 'xrotation', 'yrotation'
+                          'zrotation', 'invert_xaxis', 'invert_yaxis',
+                          'invert_zaxis']
+
     def __init__(self, overlay, ranges=None, **params):
         if 'projection' not in params:
             params['projection'] = self._get_projection(overlay)
@@ -735,6 +819,7 @@ class OverlayPlot(LegendPlot, GenericOverlayPlot):
         self.handles['legend_data'] = data
 
 
+    @mpl_rc_context
     def initialize_plot(self, ranges=None):
         axis = self.handles['axis']
         key = self.keys[-1]
@@ -750,28 +835,32 @@ class OverlayPlot(LegendPlot, GenericOverlayPlot):
         if self.show_legend:
             self._adjust_legend(element, axis)
 
-        return self._finalize_axis(key, ranges=ranges, title=self._format_title(key))
+        return self._finalize_axis(key, element=element, ranges=ranges,
+                                   title=self._format_title(key))
 
 
+    @mpl_rc_context
     def update_frame(self, key, ranges=None, element=None):
         axis = self.handles['axis']
-        if element is None:
+        reused = isinstance(self.hmap, DynamicMap) and self.overlaid
+        if element is None and not reused:
             element = self._get_frame(key)
-        else:
+        elif element is not None:
             self.current_frame = element
             self.current_key = key
+        empty = element is None
 
         if isinstance(self.hmap, DynamicMap):
             range_obj = element
-            items = element.items()
         else:
             range_obj = self.hmap
-            items = element.items()
-        ranges = self.compute_ranges(range_obj, key, ranges)
+        items = [] if element is None else element.items()
 
+        if not empty:
+            ranges = self.compute_ranges(range_obj, key, ranges)
         for k, subplot in self.subplots.items():
-            el = element.get(k, None)
-            if isinstance(self.hmap, DynamicMap):
+            el = None if empty else element.get(k, None)
+            if isinstance(self.hmap, DynamicMap) and not empty:
                 idx = dynamic_update(self, subplot, k, element, items)
                 if idx is not None:
                     _, el = items.pop(idx)
@@ -782,7 +871,7 @@ class OverlayPlot(LegendPlot, GenericOverlayPlot):
                             "were not initialized correctly and could not be "
                             "rendered.")
 
-        if self.show_legend:
+        if self.show_legend and not empty:
             self._adjust_legend(element, axis)
 
-        self._finalize_axis(key, ranges=ranges)
+        self._finalize_axis(key, element=element, ranges=ranges)
