@@ -1,6 +1,7 @@
 from __future__ import division
 
 from itertools import chain
+from contextlib import contextmanager
 
 import numpy as np
 import matplotlib as mpl
@@ -13,13 +14,35 @@ from ...core import (OrderedDict, HoloMap, AdjointLayout, NdLayout,
                      Collator, GridMatrix, Layout)
 from ...core.options import Store, Compositor, SkipRendering
 from ...core.util import int_to_roman, int_to_alpha, basestring
-from ...core import traversal
-from ..plot import DimensionedPlot, GenericLayoutPlot, GenericCompositePlot
-from ..util import get_dynamic_mode, initialize_sampled
-from .renderer import MPLRenderer
+from ..plot import (DimensionedPlot, GenericLayoutPlot, GenericCompositePlot,
+                    GenericElementPlot)
+from ..util import attach_streams
 from .util import compute_ratios, fix_aspect
 
 
+@contextmanager
+def _rc_context(rcparams):
+    """
+    Context manager that temporarily overrides the pyplot rcParams.
+    """
+    old_rcparams = plt.rcParams.copy()
+    plt.rcParams.update(rcparams)
+    try:
+        yield
+    finally:
+        plt.rcParams = old_rcparams
+
+def mpl_rc_context(f):
+    """
+    Decorator for MPLPlot methods applying the matplotlib rc params
+    in the plots fig_rcparams while when method is called.
+    """
+    def wrapper(self, *args, **kwargs):
+        with _rc_context(self.fig_rcparams):
+            return f(self, *args, **kwargs)
+    return wrapper
+
+
 class MPLPlot(DimensionedPlot):
     """
     An MPLPlot object draws a matplotlib figure object when called or
@@ -30,7 +53,8 @@ class MPLPlot(DimensionedPlot):
     via the anim() method.
     """
 
-    renderer = MPLRenderer
+    backend = 'matplotlib'
+
     sideplots = {}
 
     fig_alpha = param.Number(default=1.0, bounds=(0, 1), doc="""
@@ -105,7 +129,18 @@ class MPLPlot(DimensionedPlot):
                                for i in self.fig_inches]
         else:
             self.fig_inches *= self.fig_scale
-        fig, axis = self._init_axis(fig, axis)
+        if self.fig_latex:
+            self.fig_rcparams['text.usetex'] = True
+
+        if self.renderer.interactive:
+            plt.ion()
+            self._close_figures = False
+        elif not self.renderer.notebook_context:
+            plt.ioff()
+
+        with mpl.rc_context(rc=self.fig_rcparams):
+            fig, axis = self._init_axis(fig, axis)
+
         self.handles['fig'] = fig
         self.handles['axis'] = axis
 
@@ -122,27 +157,22 @@ class MPLPlot(DimensionedPlot):
         a new figure.
         """
         if not fig and self._create_fig:
-            rc_params = self.fig_rcparams
-            if self.fig_latex:
-                rc_params['text.usetex'] = True
-            with mpl.rc_context(rc=rc_params):
-                fig = plt.figure()
-                l, b, r, t = self.fig_bounds
-                inches = self.fig_inches
-                fig.subplots_adjust(left=l, bottom=b, right=r, top=t)
-                fig.patch.set_alpha(self.fig_alpha)
-                if isinstance(inches, (tuple, list)):
-                    inches = list(inches)
-                    if inches[0] is None:
-                        inches[0] = inches[1]
-                    elif inches[1] is None:
-                        inches[1] = inches[0]
-                    fig.set_size_inches(list(inches))
-                else:
-                    fig.set_size_inches([inches, inches])
-                axis = fig.add_subplot(111, projection=self.projection)
-                axis.set_aspect('auto')
-
+            fig = plt.figure()
+            l, b, r, t = self.fig_bounds
+            inches = self.fig_inches
+            fig.subplots_adjust(left=l, bottom=b, right=r, top=t)
+            fig.patch.set_alpha(self.fig_alpha)
+            if isinstance(inches, (tuple, list)):
+                inches = list(inches)
+                if inches[0] is None:
+                    inches[0] = inches[1]
+                elif inches[1] is None:
+                    inches[1] = inches[0]
+                fig.set_size_inches(list(inches))
+            else:
+                fig.set_size_inches([inches, inches])
+            axis = fig.add_subplot(111, projection=self.projection)
+            axis.set_aspect('auto')
         return fig, axis
 
 
@@ -207,11 +237,8 @@ class MPLPlot(DimensionedPlot):
         if self._close_figures: plt.close(figure)
         return anim
 
+
     def update(self, key):
-        rc_params = self.fig_rcparams
-        if self.fig_latex:
-            rc_params['text.usetex'] = True
-        mpl.rcParams.update(rc_params)
         if len(self) == 1 and key == 0 and not self.drawn:
             return self.initialize_plot()
         return self.__getitem__(key)
@@ -224,6 +251,7 @@ class CompositePlot(GenericCompositePlot, MPLPlot):
     subplots to form a Layout.
     """
 
+    @mpl_rc_context
     def update_frame(self, key, ranges=None):
         ranges = self.compute_ranges(self.layout, key, ranges)
         for subplot in self.subplots.values():
@@ -281,23 +309,11 @@ class GridPlot(CompositePlot):
         Rotation angle of the yticks.""")
 
     def __init__(self, layout, axis=None, create_axes=True, ranges=None,
-                 keys=None, dimensions=None, layout_num=1, **params):
+                 layout_num=1, keys=None, **params):
         if not isinstance(layout, GridSpace):
             raise Exception("GridPlot only accepts GridSpace.")
-        self.layout = layout
-        self.cols, self.rows = layout.shape
-        self.layout_num = layout_num
-        extra_opts = self.lookup_options(layout, 'plot').options
-        if not keys or not dimensions:
-            dimensions, keys = traversal.unique_dimkeys(layout)
-        if 'uniform' not in params:
-            params['uniform'] = traversal.uniform(layout)
-        dynamic, sampled = get_dynamic_mode(layout)
-        if sampled:
-            initialize_sampled(layout, dimensions, keys[0])
-        super(GridPlot, self).__init__(keys=keys, dimensions=dimensions,
-                                       dynamic=dynamic,
-                                       **dict(extra_opts, **params))
+        super(GridPlot, self).__init__(layout, layout_num=layout_num,
+                                       ranges=ranges, keys=keys, **params)
         # Compute ranges layoutwise
         grid_kwargs = {}
         if axis is not None:
@@ -306,10 +322,18 @@ class GridPlot(CompositePlot):
             grid_kwargs = {'left': l, 'right': l+w, 'bottom': b, 'top': b+h}
             self.position = (l, b, w, h)
 
+        self.cols, self.rows = layout.shape
         self.fig_inches = self._get_size()
         self._layoutspec = gridspec.GridSpec(self.rows, self.cols, **grid_kwargs)
-        self.subplots, self.subaxes, self.layout = self._create_subplots(layout, axis, ranges, create_axes)
 
+        with mpl.rc_context(rc=self.fig_rcparams):
+            self.subplots, self.subaxes, self.layout = self._create_subplots(layout, axis,
+                                                                             ranges, create_axes)
+        if self.top_level:
+            self.comm = self.init_comm()
+            self.traverse(lambda x: setattr(x, 'comm', self.comm))
+            self.traverse(lambda x: attach_streams(self, x.hmap, 2),
+                          [GenericElementPlot])
 
     def _get_size(self):
         max_dim = max(self.layout.shape)
@@ -405,7 +429,7 @@ class GridPlot(CompositePlot):
                               dimensions=self.dimensions, show_title=False,
                               subplot=not create_axes, ranges=frame_ranges,
                               uniform=self.uniform, keys=self.keys,
-                              show_legend=False)
+                              show_legend=False, renderer=self.renderer)
                 plotting_class = Store.registry['matplotlib'][vtype]
                 subplot = plotting_class(view,  **dict(opts, **dict(params, **kwargs)))
                 collapsed_layout[coord] = subplot.layout if isinstance(subplot, CompositePlot) else subplot.hmap
@@ -424,6 +448,7 @@ class GridPlot(CompositePlot):
         return subplots, subaxes, collapsed_layout
 
 
+    @mpl_rc_context
     def initialize_plot(self, ranges=None):
         # Get the extent of the layout elements (not the whole layout)
         key = self.keys[-1]
@@ -464,14 +489,15 @@ class GridPlot(CompositePlot):
             layout_axis.set_position(self.position)
         layout_axis.patch.set_visible(False)
 
-        tick_fontsize = self._fontsize('ticks','labelsize',common=False)
-        if tick_fontsize: layout_axis.tick_params(**tick_fontsize)
+        for ax, ax_obj in zip(['x', 'y'], [layout_axis.xaxis, layout_axis.yaxis]):
+            tick_fontsize = self._fontsize('%sticks' % ax,'labelsize', common=False)
+            if tick_fontsize: ax_obj.set_tick_params(**tick_fontsize)
 
         # Set labels
-        layout_axis.set_xlabel(str(layout.kdims[0]),
+        layout_axis.set_xlabel(layout.kdims[0].pprint_label,
                                **self._fontsize('xlabel'))
         if layout.ndims == 2:
-            layout_axis.set_ylabel(str(layout.kdims[1]),
+            layout_axis.set_ylabel(layout.kdims[1].pprint_label,
                                **self._fontsize('ylabel'))
 
         # Compute and set x- and y-ticks
@@ -483,7 +509,7 @@ class GridPlot(CompositePlot):
             layout_axis.get_yaxis().set_visible(False)
         else:
             dim1_keys, dim2_keys = zip(*keys)
-            layout_axis.set_ylabel(str(dims[1]))
+            layout_axis.set_ylabel(dims[1].pprint_label)
             layout_axis.set_aspect(float(self.rows)/self.cols)
 
         # Process ticks
@@ -566,7 +592,7 @@ class GridPlot(CompositePlot):
 
 
 
-class AdjointLayoutPlot(CompositePlot):
+class AdjointLayoutPlot(MPLPlot):
     """
     LayoutPlot allows placing up to three Views in a number of
     predefined and fixed layouts, which are defined by the layout_dict
@@ -596,6 +622,7 @@ class AdjointLayoutPlot(CompositePlot):
         super(AdjointLayoutPlot, self).__init__(subplots=subplots, **params)
 
 
+    @mpl_rc_context
     def initialize_plot(self, ranges=None):
         """
         Plot all the views contained in the AdjointLayout Object using axes
@@ -667,6 +694,7 @@ class AdjointLayoutPlot(CompositePlot):
                 ax.set_aspect('equal')
 
 
+    @mpl_rc_context
     def update_frame(self, key, ranges=None):
         for pos in self.view_positions:
             subplot = self.subplots.get(pos)
@@ -709,20 +737,24 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
       Specifies the space between horizontally adjacent elements in the grid.
       Default value is set conservatively to avoid overlap of subplots.""")
 
-    vspace = param.Number(default=0.1, doc="""
+    vspace = param.Number(default=0.3, doc="""
       Specifies the space between vertically adjacent elements in the grid.
       Default value is set conservatively to avoid overlap of subplots.""")
 
     fontsize = param.Parameter(default={'title':16}, allow_None=True)
 
     # Whether to enable fix for non-square figures
-    # Will be enabled by default in v1.7
-    # If enabled default vspace should be increased to 0.3
-    v17_layout_format = False
+    v17_layout_format = True
 
-    def __init__(self, layout, **params):
-        super(LayoutPlot, self).__init__(layout=layout, **params)
-        self.subplots, self.subaxes, self.layout = self._compute_gridspec(layout)
+    def __init__(self, layout, keys=None, **params):
+        super(LayoutPlot, self).__init__(layout=layout, keys=keys, **params)
+        with mpl.rc_context(rc=self.fig_rcparams):
+            self.subplots, self.subaxes, self.layout = self._compute_gridspec(layout)
+        if self.top_level:
+            self.comm = self.init_comm()
+            self.traverse(lambda x: setattr(x, 'comm', self.comm))
+            self.traverse(lambda x: attach_streams(self, x.hmap, 2),
+                          [GenericElementPlot])
 
 
     def _compute_gridspec(self, layout):
@@ -743,7 +775,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
         col_widthratios, row_heightratios = {}, {}
         for (r, c) in self.coords:
             # Get view at layout position and wrap in AdjointLayout
-            _, view = layout_items.get((r, c), (None, None))
+            _, view = layout_items.get((c, r) if self.transpose else (r, c), (None, None))
             layout_view = view if isinstance(view, AdjointLayout) else AdjointLayout([view])
             layouts[(r, c)] = layout_view
 
@@ -889,6 +921,11 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             empty = isinstance(obj.main, Empty)
             if empty:
                 obj = AdjointLayout([])
+            elif view is None or not view.traverse(lambda x: x, [Element]):
+                self.warning('%s is empty, skipping subplot.' % obj.main)
+                continue
+            elif self.transpose:
+                layout_count = (c*self.rows+(r+1))
             else:
                 layout_count += 1
             subaxes = [plt.subplot(self.gs[ind], projection=proj)
@@ -973,7 +1010,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             # Pos will be one of 'main', 'top' or 'right' or None
             view = layout.get(pos, None)
             ax = axes.get(pos, None)
-            if view is None:
+            if view is None or not view.traverse(lambda x: x, [Element]):
                 projections.append(None)
                 continue
 
@@ -993,8 +1030,6 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
                 own_params = self.get_param_values(onlychanged=True)
                 sublabel_opts = {k: v for k, v in own_params
                                  if 'sublabel_' in k}
-                if not isinstance(view, GridSpace):
-                    override_opts = dict(aspect='square')
             elif pos == 'right':
                 right_opts = dict(invert_axes=True,
                                   xaxis=None)
@@ -1018,7 +1053,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
                                       layout_dimensions=layout_dimensions,
                                       ranges=ranges, subplot=True,
                                       uniform=self.uniform, layout_num=num,
-                                      **plotopts)
+                                      renderer=self.renderer, **plotopts)
             if isinstance(view, (Element, HoloMap, Collator, CompositeOverlay)):
                 adjoint_clone[pos] = subplots[pos].hmap
             else:
@@ -1026,6 +1061,7 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
         return subplots, adjoint_clone, projections
 
 
+    @mpl_rc_context
     def initialize_plot(self):
         key = self.keys[-1]
         ranges = self.compute_ranges(self.layout, key, None)
@@ -1046,17 +1082,18 @@ class LayoutPlot(GenericLayoutPlot, CompositePlot):
             traverse_fn = lambda x: x.handles.get('bbox_extra_artists', None)
             extra_artists = list(chain(*[artists for artists in self.traverse(traverse_fn)
                                          if artists is not None]))
-            aspect = fix_aspect(fig, self.rows, self.cols,
-                                title_obj, extra_artists,
-                                vspace=self.vspace*self.fig_scale,
-                                hspace=self.hspace*self.fig_scale)
+            fix_aspect(fig, self.rows, self.cols,
+                       title_obj, extra_artists,
+                       vspace=self.vspace*self.fig_scale,
+                       hspace=self.hspace*self.fig_scale)
             colorbars = self.traverse(specs=[lambda x: hasattr(x, 'colorbar')])
             for cbar_plot in colorbars:
                 if cbar_plot.colorbar:
                     cbar_plot._draw_colorbar(redraw=False)
             adjoined = self.traverse(specs=[AdjointLayoutPlot])
             for adjoined in adjoined:
-                adjoined.adjust_positions(redraw=False)
+                if len(adjoined.subplots) > 1:
+                    adjoined.adjust_positions(redraw=False)
         return self._finalize_axis(None)
 
 
