@@ -18,8 +18,11 @@ from ..core.layout import Empty, NdLayout, Layout
 from ..core.options import Store, Compositor, SkipRendering
 from ..core.overlay import NdOverlay
 from ..core.spaces import HoloMap, DynamicMap
+from ..core.util import stream_parameters
 from ..element import Table
-from .util import get_dynamic_mode, initialize_sampled, dim_axis_label
+from .util import (get_dynamic_mode, initialize_unbounded, dim_axis_label,
+                   attach_streams, traverse_setter, get_nested_streams,
+                   compute_overlayable_zorders, get_plot_frame)
 
 
 class Plot(param.Parameterized):
@@ -68,12 +71,12 @@ class Plot(param.Parameterized):
     @classmethod
     def lookup_options(cls, obj, group):
         try:
-            plot_class = cls.renderer.plotting_class(obj)
+            plot_class = Store.renderers[cls.backend].plotting_class(obj)
             style_opts = plot_class.style_opts
         except SkipRendering:
             style_opts = None
 
-        node = Store.lookup_options(cls.renderer.backend, obj, group)
+        node = Store.lookup_options(cls.backend, obj, group)
         if group == 'style' and style_opts:
             return node.filtered(style_opts)
         else:
@@ -156,7 +159,8 @@ class DimensionedPlot(Plot):
 
     #Allowed fontsize keys
     _fontsize_keys = ['xlabel','ylabel', 'labels', 'ticks',
-                      'title', 'legend', 'legend_title']
+                      'title', 'legend', 'legend_title', 'xticks',
+                      'yticks']
 
     show_title = param.Boolean(default=True, doc="""
         Whether to display the plot title.""")
@@ -178,7 +182,7 @@ class DimensionedPlot(Plot):
 
     def __init__(self, keys=None, dimensions=None, layout_dimensions=None,
                  uniform=True, subplot=False, adjoined=None, layout_num=0,
-                 style=None, subplots=None, dynamic=False, **params):
+                 style=None, subplots=None, dynamic=False, renderer=None, **params):
         self.subplots = subplots
         self.adjoined = adjoined
         self.dimensions = dimensions
@@ -195,6 +199,11 @@ class DimensionedPlot(Plot):
         self.current_frame = None
         self.current_key = None
         self.ranges = {}
+        self.renderer = renderer if renderer else Store.renderers[self.backend].instance()
+        self.comm = None
+        self._force = False
+        self._updated = False # Whether the plot should be marked as updated
+
         params = {k: v for k, v in params.items()
                   if k in self.params()}
         super(DimensionedPlot, self).__init__(**params)
@@ -204,10 +213,10 @@ class DimensionedPlot(Plot):
         """
         Get the state of the Plot for a given frame number.
         """
-        if not self.dynamic == 'open' and isinstance(frame, int) and frame > len(self):
+        if isinstance(frame, int) and frame > len(self):
             self.warning("Showing last frame available: %d" % len(self))
         if not self.drawn: self.handles['fig'] = self.initialize_plot()
-        if not self.dynamic == 'open' and not isinstance(frame, tuple):
+        if not isinstance(frame, tuple):
             frame = self.keys[frame]
         self.update_frame(frame)
         return self.state
@@ -260,8 +269,6 @@ class DimensionedPlot(Plot):
         Returns the formatted dimension group strings
         for a particular frame.
         """
-        if self.dynamic == 'open' and self.current_key:
-            key = self.current_key
         if self.layout_dimensions is not None:
             dimensions, key = zip(*self.layout_dimensions.items())
         elif not self.dynamic and (not self.uniform or len(self) == 1) or self.subplot:
@@ -273,7 +280,7 @@ class DimensionedPlot(Plot):
                             zip(dimensions, key)]
         groups = [', '.join(dimension_labels[i*group_size:(i+1)*group_size])
                   for i in range(len(dimension_labels))]
-        return util.safe_unicode(separator.join(g for g in groups if g))
+        return util.bytes_to_unicode(separator.join(g for g in groups if g))
 
 
     def _fontsize(self, key, label='fontsize', common=True):
@@ -292,12 +299,12 @@ class DimensionedPlot(Plot):
             return {label:self.fontsize[key]}
         elif key in ['ylabel', 'xlabel'] and 'labels' in self.fontsize:
             return {label:self.fontsize['labels']}
+        elif key in ['xticks', 'yticks'] and 'ticks' in self.fontsize:
+            return {label:self.fontsize['ticks']}
         else:
             return {}
 
 
-
-
     def compute_ranges(self, obj, key, ranges):
         """
         Given an object, a specific key and the normalization options
@@ -325,7 +332,7 @@ class DimensionedPlot(Plot):
             elements = []
             # Skip if ranges are cached or already computed by a
             # higher-level container object.
-            framewise = framewise or self.dynamic
+            framewise = framewise or self.dynamic or len(elements) == 1
             if group in ranges and (not framewise or ranges is not self.ranges):
                 continue
             elif not framewise: # Traverse to get all elements
@@ -333,8 +340,10 @@ class DimensionedPlot(Plot):
             elif key is not None: # Traverse to get elements for each frame
                 frame = self._get_frame(key)
                 elements = [] if frame is None else frame.traverse(return_fn, [group])
-            if not axiswise or ((not framewise or len(elements) == 1)
-                                and isinstance(obj, HoloMap)): # Compute new ranges
+            # Only compute ranges if not axiswise on a composite plot
+            # or not framewise on a Overlay or ElementPlot
+            if (not (axiswise and not isinstance(obj, HoloMap)) or
+                (not framewise and isinstance(obj, HoloMap))):
                 self._compute_group_range(group, elements, ranges)
         self.ranges.update(ranges)
         return ranges
@@ -420,7 +429,7 @@ class DimensionedPlot(Plot):
             selected = {o: options.options[o]
                         for o in opts if o in options.options}
             if opt_type == 'plot' and defaults:
-                plot = Store.registry[cls.renderer.backend].get(type(x))
+                plot = Store.registry[cls.backend].get(type(x))
                 selected['defaults'] = {o: getattr(plot, o) for o in opts
                                         if o not in selected and hasattr(plot, o)}
             key = keyfn(x) if keyfn else None
@@ -469,7 +478,51 @@ class DimensionedPlot(Plot):
     def update(self, key):
         if len(self) == 1 and key == 0 and not self.drawn:
             return self.initialize_plot()
-        return self.__getitem__(key)
+        item = self.__getitem__(key)
+        self.traverse(lambda x: setattr(x, '_updated', True))
+        return item
+
+
+    def refresh(self, **kwargs):
+        """
+        Refreshes the plot by rerendering it and then pushing
+        the updated data if the plot has an associated Comm.
+        """
+        traverse_setter(self, '_force', True)
+        key = self.current_key if self.current_key else self.keys[0]
+        dim_streams = [stream for stream in self.streams
+                       if any(c in self.dimensions for c in stream.contents)]
+        stream_params = stream_parameters(dim_streams)
+        key = tuple(None if d in stream_params else k
+                    for d, k in zip(self.dimensions, key))
+        stream_key = util.wrap_tuple_streams(key, self.dimensions, self.streams)
+
+        # Update if not top-level, batched or an ElementPlot
+        if not self.top_level or isinstance(self, GenericElementPlot):
+            self.update(stream_key)
+
+        if self.comm is not None and self.top_level:
+            self.push()
+
+
+    def push(self):
+        """
+        Pushes updated plot data via the Comm.
+        """
+        if self.comm is None:
+            raise Exception('Renderer does not have a comm.')
+        diff = self.renderer.diff(self)
+        self.comm.send(diff)
+
+
+    def init_comm(self):
+        """
+        Initializes comm and attaches streams.
+        """
+        comm = None
+        if self.dynamic or self.renderer.widget_mode == 'live':
+            comm = self.renderer.comms[self.renderer.mode][0](self)
+        return comm
 
 
     def __len__(self):
@@ -499,12 +552,18 @@ class GenericElementPlot(DimensionedPlot):
     # plot and a 'batched' method to draw multiple Elements at once
     _plot_methods = {}
 
+    # Declares the options that are propagated from sub-elements of the
+    # plot, mostly useful for inheriting options from individual
+    # Elements on an OverlayPlot. Enabled by default in v1.7.
+    _propagate_options = []
+    v17_option_propagation = True
+
     def __init__(self, element, keys=None, ranges=None, dimensions=None,
                  batched=False, overlaid=0, cyclic_index=0, zorder=0, style=None,
-                 overlay_dims={}, **params):
+                 overlay_dims={}, stream_sources=[], **params):
         self.zorder = zorder
         self.cyclic_index = cyclic_index
-        self.overlaid = overlaid and not batched
+        self.overlaid = overlaid
         self.batched = batched
         self.overlay_dims = overlay_dims
 
@@ -514,19 +573,43 @@ class GenericElementPlot(DimensionedPlot):
         else:
             self.hmap = element
 
+        if overlaid:
+            self.stream_sources = stream_sources
+        else:
+            self.stream_sources = compute_overlayable_zorders(self.hmap)
+
         plot_element = self.hmap.last
-        if self.batched:
-            plot_element = plot_element.last
+        if self.batched and not isinstance(self, GenericOverlayPlot):
+            plot_element = [el for el in plot_element if el][-1]
+
+        self.top_level = keys is None
+        if self.top_level:
+            dimensions = self.hmap.kdims
+            keys = list(self.hmap.data.keys())
 
         self.style = self.lookup_options(plot_element, 'style') if style is None else style
-        dimensions = self.hmap.kdims if dimensions is None else dimensions
-        keys = keys if keys else list(self.hmap.data.keys())
         plot_opts = self.lookup_options(plot_element, 'plot').options
+        if self.v17_option_propagation:
+            inherited = self._traverse_options(plot_element, 'plot',
+                                               self._propagate_options,
+                                               defaults=False)
+            plot_opts.update(**{k: v[0] for k, v in inherited.items()})
 
-        dynamic = False if not isinstance(element, DynamicMap) or element.sampled else element.mode
+        dynamic = isinstance(element, DynamicMap) and not element.unbounded
         super(GenericElementPlot, self).__init__(keys=keys, dimensions=dimensions,
                                                  dynamic=dynamic,
                                                  **dict(params, **plot_opts))
+        streams = []
+        if isinstance(self.hmap, DynamicMap):
+            streams = get_nested_streams(self.hmap)
+        self.streams = streams
+        if self.top_level:
+            self.comm = self.init_comm()
+            self.traverse(lambda x: setattr(x, 'comm', self.comm))
+
+        # Attach streams if not overlaid and not a batched ElementPlot
+        if not (self.overlaid or (self.batched and not isinstance(self, GenericOverlayPlot))):
+            attach_streams(self, self.hmap)
 
         # Update plot and style options for batched plots
         if self.batched:
@@ -545,58 +628,33 @@ class GenericElementPlot(DimensionedPlot):
         taking into account possible batching of elements.
         """
         spec = util.get_overlay_spec(overlay, key, el)
-        try:
-            return self.ordering.index(spec)
-        except ValueError:
-            self.ordering = sorted(self.ordering+[spec])
-            return self.ordering.index(spec)
+        return self.ordering.index(spec)
+
+
+    def _updated_zorders(self, overlay):
+        specs = [util.get_overlay_spec(overlay, key, el)
+                 for key, el in overlay.data.items()]
+        self.ordering = sorted(set(self.ordering+specs))
+        return [self.ordering.index(spec) for spec in specs]
 
 
     def _get_frame(self, key):
         if isinstance(self.hmap, DynamicMap) and self.overlaid and self.current_frame:
             self.current_key = key
             return self.current_frame
-        elif self.dynamic:
-            key, frame = util.get_dynamic_item(self.hmap, self.dimensions, key)
-            if not isinstance(key, tuple): key = (key,)
-            key_map = dict(zip([d.name for d in self.hmap.kdims], key))
-            key = tuple(key_map.get(d.name, None) for d in self.dimensions)
-            if not key in self.keys:
-                self.keys.append(key)
-            self.current_frame = frame
-            self.current_key = key
-            return frame
-
-        if isinstance(key, int):
-            key = self.hmap.keys()[min([key, len(self.hmap)-1])]
-
-        if key == self.current_key:
+        elif key == self.current_key and not self._force:
             return self.current_frame
-        else:
-            self.current_key = key
 
-        if self.uniform:
-            if not isinstance(key, tuple): key = (key,)
-            kdims = [d.name for d in self.hmap.kdims]
-            if self.dimensions is None:
-                dimensions = kdims
-            else:
-                dimensions = [d.name for d in self.dimensions]
-            if kdims == ['Frame'] and kdims != dimensions:
-                select = dict(Frame=0)
-            else:
-                select = {d: key[dimensions.index(d)]
-                          for d in kdims}
-        else:
-            select = dict(zip(self.hmap.dimensions('key', label=True), key))
-        try:
-            selection = self.hmap.select((HoloMap, DynamicMap), **select)
-        except KeyError:
-            selection = None
-        selection = selection.last if isinstance(selection, HoloMap) else selection
-        self.current_frame = selection
+        cached = self.current_key is None
+        key_map = dict(zip([d.name for d in self.dimensions], key))
+        frame = get_plot_frame(self.hmap, key_map, cached)
+        traverse_setter(self, '_force', False)
 
-        return selection
+        if not key in self.keys:
+            self.keys.append(key)
+        self.current_frame = frame
+        self.current_key = key
+        return frame
 
 
     def get_extents(self, view, ranges):
@@ -640,17 +698,27 @@ class GenericElementPlot(DimensionedPlot):
                 extents = util.max_extents(extent_list, self.projection == '3d')
         else:
             extents = (np.NaN,) * num
-        return tuple(l1 if l2 is None or not np.isfinite(l2) else
-                     l2 for l1, l2 in zip(range_extents, extents))
+
+        if getattr(self, 'shared_axes', False) and self.subplot:
+            return util.max_extents([range_extents, extents], self.projection == '3d')
+        else:
+            max_extent = []
+            for l1, l2 in zip(range_extents, extents):
+                if (isinstance(l2, util.datetime_types)
+                    or (l2 is not None and np.isfinite(l2))):
+                    max_extent.append(l2)
+                else:
+                    max_extent.append(l1)
+            return tuple(max_extent)
 
 
     def _get_axis_labels(self, dimensions, xlabel=None, ylabel=None, zlabel=None):
         if dimensions and xlabel is None:
-            xlabel = dim_axis_label(dimensions[0])
+            xlabel = dim_axis_label(dimensions[0]) if dimensions[0] else ''
         if len(dimensions) >= 2 and ylabel is None:
-            ylabel = dim_axis_label(dimensions[1])
+            ylabel = dim_axis_label(dimensions[1]) if dimensions[1] else ''
         if self.projection == '3d' and len(dimensions) >= 3 and zlabel is None:
-            zlabel = dim_axis_label(dimensions[2])
+            zlabel = dim_axis_label(dimensions[2]) if dimensions[2] else ''
         return xlabel, ylabel, zlabel
 
 
@@ -665,9 +733,9 @@ class GenericElementPlot(DimensionedPlot):
         if self.layout_dimensions:
             title = dim_title
         else:
-            title_format = util.safe_unicode(self.title_format)
-            title = title_format.format(label=util.safe_unicode(label),
-                                        group=util.safe_unicode(group),
+            title_format = util.bytes_to_unicode(self.title_format)
+            title = title_format.format(label=util.bytes_to_unicode(label),
+                                        group=util.bytes_to_unicode(group),
                                         type=type_name,
                                         dimensions=dim_title)
         return title.strip(' \n')
@@ -711,14 +779,21 @@ class GenericOverlayPlot(GenericElementPlot):
 
     _passed_handles = []
 
-    def __init__(self, overlay, ranges=None, batched=True, **params):
-        super(GenericOverlayPlot, self).__init__(overlay, ranges=ranges,
+    def __init__(self, overlay, ranges=None, batched=True, keys=None, **params):
+        super(GenericOverlayPlot, self).__init__(overlay, ranges=ranges, keys=keys,
                                                  batched=batched, **params)
 
         # Apply data collapse
         self.hmap = Compositor.collapse(self.hmap, None, mode='data')
         self.hmap = self._apply_compositor(self.hmap, ranges, self.keys)
         self.subplots = self._create_subplots(ranges)
+        self.traverse(lambda x: setattr(x, 'comm', self.comm))
+        self.top_level = keys is None
+        if self.top_level:
+            self.comm = self.init_comm()
+            self.traverse(lambda x: setattr(x, 'comm', self.comm))
+            self.traverse(lambda x: attach_streams(self, x.hmap, 2),
+                          [GenericElementPlot])
 
 
     def _apply_compositor(self, holomap, ranges=None, keys=None, dimensions=None):
@@ -738,7 +813,7 @@ class GenericOverlayPlot(GenericElementPlot):
         else:
             mapwise_ranges = self.compute_ranges(holomap, None, None)
             frame_ranges = OrderedDict([(key, self.compute_ranges(holomap, key, mapwise_ranges))
-                                        for key in holomap.keys()])
+                                        for key in holomap.data.keys()])
         ranges = frame_ranges.values()
 
         return Compositor.collapse(holomap, (ranges, frame_ranges.keys()), mode='display')
@@ -749,12 +824,16 @@ class GenericOverlayPlot(GenericElementPlot):
         ordering = util.layer_sort(self.hmap)
         registry = Store.registry[self.renderer.backend]
         batched = self.batched and type(self.hmap.last) is NdOverlay
+        stream_sources = self.stream_sources
         if batched:
             batchedplot = registry.get(type(self.hmap.last.last))
         if (batched and batchedplot and 'batched' in batchedplot._plot_methods and
             (not self.show_legend or len(ordering) > self.legend_limit)):
             self.batched = True
             keys, vmaps = [()], [self.hmap]
+            param_vals = dict(self.get_param_values())
+            propagate = {opt: param_vals[opt] for opt in self._propagate_options
+                         if opt in param_vals}
         else:
             self.batched = False
             keys, vmaps = self.hmap.split_overlays()
@@ -771,13 +850,13 @@ class GenericOverlayPlot(GenericElementPlot):
 
         subplots = OrderedDict()
         for (key, vmap) in zip(keys, vmaps):
-            opts = {}
+            opts = {'overlaid': overlay_type}
             if self.hmap.type == Overlay:
                 style_key = (vmap.type.__name__,) + key
             else:
                 if not isinstance(key, tuple): key = (key,)
                 style_key = group_fn(vmap) + key
-                opts['overlay_dims'] = OrderedDict(zip(vmap.last.kdims, key))
+                opts['overlay_dims'] = OrderedDict(zip(self.hmap.last.kdims, key))
 
             if self.batched:
                 vtype = type(vmap.last.last)
@@ -801,20 +880,27 @@ class GenericOverlayPlot(GenericElementPlot):
 
             if issubclass(plottype, GenericOverlayPlot):
                 opts['show_legend'] = self.show_legend
-            elif self.batched:
+                if not any(len(frame) for frame in vmap):
+                    self.warning('%s is empty and will be skipped during plotting'
+                                 % vmap.last)
+                    continue
+            elif self.batched and 'batched' in plottype._plot_methods:
                 opts['batched'] = self.batched
+                opts['overlaid'] = self.overlaid
+                opts.update(propagate)
             if len(ordering) > self.legend_limit:
                 opts['show_legend'] = False
             style = self.lookup_options(vmap.last, 'style').max_cycles(group_length)
+            passed_handles = {k: v for k, v in self.handles.items()
+                              if k in self._passed_handles}
             plotopts = dict(opts, cyclic_index=cyclic_index,
                             invert_axes=self.invert_axes,
                             dimensions=self.dimensions, keys=self.keys,
                             layout_dimensions=self.layout_dimensions,
-                            overlaid=overlay_type, ranges=ranges,
-                            show_title=self.show_title, style=style,
-                            uniform=self.uniform, zorder=zorder,
-                            **{k: v for k, v in self.handles.items()
-                               if k in self._passed_handles})
+                            ranges=ranges, show_title=self.show_title,
+                            style=style, uniform=self.uniform,
+                            renderer=self.renderer, stream_sources=stream_sources,
+                            zorder=zorder, **passed_handles)
 
             if not isinstance(key, tuple): key = (key,)
             subplots[key] = plottype(vmap, **plotopts)
@@ -830,8 +916,8 @@ class GenericOverlayPlot(GenericElementPlot):
     def get_extents(self, overlay, ranges):
         extents = []
         items = overlay.items()
-        if self.batched:
-            subplot = self.subplots.values()[0]
+        if self.batched and self.subplots:
+            subplot = list(self.subplots.values())[0]
             subplots = [(k, subplot) for k in overlay.data.keys()]
         else:
             subplots = self.subplots.items()
@@ -857,45 +943,46 @@ class GenericOverlayPlot(GenericElementPlot):
 
 class GenericCompositePlot(DimensionedPlot):
 
+    def __init__(self, layout, keys=None, dimensions=None, **params):
+        if 'uniform' not in params:
+            params['uniform'] = traversal.uniform(layout)
+
+        self.top_level = keys is None
+        if self.top_level:
+            dimensions, keys = traversal.unique_dimkeys(layout)
+
+        dynamic, unbounded = get_dynamic_mode(layout)
+        if unbounded:
+            initialize_unbounded(layout, dimensions, keys[0])
+        self.layout = layout
+        super(GenericCompositePlot, self).__init__(keys=keys,
+                                                   dynamic=dynamic,
+                                                   dimensions=dimensions,
+                                                   **params)
+        nested_streams = layout.traverse(lambda x: get_nested_streams(x),
+                                         [DynamicMap])
+        self.streams = list(set([s for streams in nested_streams for s in streams]))
+
+
     def _get_frame(self, key):
         """
         Creates a clone of the Layout with the nth-frame for each
         Element.
         """
+        cached = self.current_key is None
         layout_frame = self.layout.clone(shared_data=False)
-        keyisint = isinstance(key, int)
-        if not isinstance(key, tuple): key = (key,)
-        nthkey_fn = lambda x: zip(tuple(x.name for x in x.kdims),
-                                  list(x.data.keys())[min([key[0], len(x)-1])])
-        if key == self.current_key:
+        if key == self.current_key and not self._force:
             return self.current_frame
         else:
             self.current_key = key
 
+        key_map = dict(zip([d.name for d in self.dimensions], key))
         for path, item in self.layout.items():
-            if self.dynamic == 'open':
-                if keyisint:
-                    counts = item.traverse(lambda x: x.counter, (DynamicMap,))
-                    if key[0] >= counts[0]:
-                        item.traverse(lambda x: next(x), (DynamicMap,))
-                    dim_keys = item.traverse(nthkey_fn, (DynamicMap,))[0]
-                else:
-                    dim_keys = zip([d.name for d in self.dimensions
-                                    if d in item.dimensions('key')], key)
-                self.current_key = tuple(k[1] for k in dim_keys)
-            elif self.uniform:
-                dim_keys = zip([d.name for d in self.dimensions
-                                if d in item.dimensions('key')], key)
-            else:
-                dim_keys = item.traverse(nthkey_fn, (HoloMap,))[0]
-            if dim_keys:
-                obj = item.select((HoloMap,), **dict(dim_keys))
-                if isinstance(obj, HoloMap) and len(obj) == 0:
-                    continue
-                else:
-                    layout_frame[path] = obj
-            else:
-                layout_frame[path] = item
+            frame = item.map(lambda x: get_plot_frame(x, key_map, cached=cached),
+                             ['DynamicMap', 'HoloMap'])
+            if frame is not None:
+                layout_frame[path] = frame
+        traverse_setter(self, '_force', False)
 
         self.current_frame = layout_frame
         return layout_frame
@@ -909,12 +996,12 @@ class GenericCompositePlot(DimensionedPlot):
         dim_title = self._frame_title(key, 3, separator)
         layout = self.layout
         type_name = type(self.layout).__name__
-        group = util.safe_unicode(layout.group if layout.group != type_name else '')
-        label = util.safe_unicode(layout.label)
-        title = util.safe_unicode(self.title_format).format(label=label,
-                                                            group=group,
-                                                            type=type_name,
-                                                            dimensions=dim_title)
+        group = util.bytes_to_unicode(layout.group if layout.group != type_name else '')
+        label = util.bytes_to_unicode(layout.label)
+        title = util.bytes_to_unicode(self.title_format).format(label=label,
+                                                                group=group,
+                                                                type=type_name,
+                                                                dimensions=dim_title)
         return title.strip(' \n')
 
 
@@ -924,24 +1011,19 @@ class GenericLayoutPlot(GenericCompositePlot):
     displays the elements in a cartesian grid in scanline order.
     """
 
+    transpose = param.Boolean(default=False, doc="""
+        Whether to transpose the layout when plotting. Switches
+        from row-based left-to-right and top-to-bottom scanline order
+        to column-based top-to-bottom and left-to-right order.""")
+
     def __init__(self, layout, **params):
         if not isinstance(layout, (NdLayout, Layout)):
             raise ValueError("GenericLayoutPlot only accepts Layout objects.")
         if len(layout.values()) == 0:
-            raise ValueError("Cannot display empty layout")
+            raise SkipRendering(warn=False)
 
-        self.layout = layout
+        super(GenericLayoutPlot, self).__init__(layout, **params)
         self.subplots = {}
-        self.rows, self.cols = layout.shape
+        self.rows, self.cols = layout.shape[::-1] if self.transpose else layout.shape
         self.coords = list(product(range(self.rows),
                                    range(self.cols)))
-        dynamic, sampled = get_dynamic_mode(layout)
-        dimensions, keys = traversal.unique_dimkeys(layout)
-        if sampled:
-            initialize_sampled(layout, dimensions, keys[0])
-
-        uniform = traversal.uniform(layout)
-        plotopts = self.lookup_options(layout, 'plot').options
-        super(GenericLayoutPlot, self).__init__(keys=keys, dimensions=dimensions,
-                                                uniform=uniform, dynamic=dynamic,
-                                                **dict(plotopts, **params))
