@@ -11,13 +11,13 @@ from contextlib import contextmanager
 import param
 from ..core.io import Exporter
 from ..core.options import Store, StoreOptions, SkipRendering
-from ..core.util import find_file, unicode
+from ..core.util import find_file, unicode, unbound_dimensions
 from .. import Layout, HoloMap, AdjointLayout
 from .widgets import NdWidget, ScrubberWidget, SelectionWidget
 
-from .. import DynamicMap
 from . import Plot
-from .util import displayable, collate
+from .comms import JupyterComm
+from .util import displayable, collate, initialize_dynamic
 
 from param.parameterized import bothmethod
 
@@ -49,8 +49,9 @@ MIME_TYPES = {
     'webm': 'video/webm',
     'mp4':  'video/mp4',
     'pdf':  'application/pdf',
-    'html':  None,
-    'json':  None
+    'html':  'text/html',
+    'json':  'text/json',
+    'server': None
 }
 
 static_template = """
@@ -82,7 +83,7 @@ class Renderer(Exporter):
         The full, lowercase name of the rendering backend or third
         part plotting package used e.g 'matplotlib' or 'cairo'.""")
 
-    dpi=param.Integer(None, allow_None=True, doc="""
+    dpi=param.Integer(None, doc="""
         The render resolution in dpi (dots per inch)""")
 
     fig = param.ObjectSelector(default='auto', objects=['auto'], doc="""
@@ -129,6 +130,11 @@ class Renderer(Exporter):
     mode_formats = {'fig': {'default': [None, 'auto']},
                     'holomap': {'default': [None, 'auto']}}
 
+    # Define comms class and message handler for each mode
+    # The Comm opens a communication channel and the message
+    # handler defines how the message is processed on the frontend
+    comms = {'default': (JupyterComm, None)}
+
     # Define appropriate widget classes
     widgets = {'scrubber': ScrubberWidget, 'widgets': SelectionWidget}
 
@@ -143,37 +149,32 @@ class Renderer(Exporter):
     # Any additional JS and CSS dependencies required by a specific backend
     backend_dependencies = {}
 
+    # Whether in a notebook context, set when running Renderer.load_nb
+    notebook_context = False
+
     def __init__(self, **params):
         self.last_plot = None
         super(Renderer, self).__init__(**params)
 
 
     @bothmethod
-    def get_plot(self_or_cls, obj):
+    def get_plot(self_or_cls, obj, renderer=None):
         """
         Given a HoloViews Viewable return a corresponding plot instance.
         """
+        # Initialize DynamicMaps with first data item
+        initialize_dynamic(obj)
+
         if not isinstance(obj, Plot) and not displayable(obj):
             obj = collate(obj)
+            initialize_dynamic(obj)
 
-        # Initialize DynamicMaps with first data item
-        dmaps = obj.traverse(lambda x: x, specs=[DynamicMap])
-        for dmap in dmaps:
-            if dmap.sampled:
-                # Skip initialization until plotting code
-                continue
-            if dmap.call_mode == 'key':
-                dmap[dmap._initial_key()]
-            else:
-                try:
-                    next(dmap)
-                except StopIteration: # Exhausted DynamicMap
-                    raise SkipRendering("DynamicMap generator exhausted.")
-
+        if not renderer: renderer = self_or_cls.instance()
         if not isinstance(obj, Plot):
             obj = Layout.from_values(obj) if isinstance(obj, AdjointLayout) else obj
             plot_opts = self_or_cls.plot_options(obj, self_or_cls.size)
-            plot = self_or_cls.plotting_class(obj)(obj, **plot_opts)
+            plot = self_or_cls.plotting_class(obj)(obj, renderer=renderer,
+                                                   **plot_opts)
             plot.update(0)
         else:
             plot = obj
@@ -187,14 +188,16 @@ class Renderer(Exporter):
         """
         if isinstance(obj, tuple(self.widgets.values())):
             return obj, 'html'
-        plot = self.get_plot(obj)
+        plot = self.get_plot(obj, renderer=self)
 
         fig_formats = self.mode_formats['fig'][self.mode]
         holomap_formats = self.mode_formats['holomap'][self.mode]
 
         if fmt in ['auto', None]:
-            if ((len(plot) == 1 and not plot.dynamic)
-                or (len(plot) > 1 and self.holomap is None)):
+            if (((len(plot) == 1 and not plot.dynamic)
+                or (len(plot) > 1 and self.holomap is None) or
+                (plot.dynamic and len(plot.keys[0]) == 0)) or
+                not unbound_dimensions(plot.streams, plot.dimensions, no_duplicates=False)):
                 fmt = fig_formats[0] if self.fig=='auto' else self.fig
             else:
                 fmt = holomap_formats[0] if self.holomap=='auto' else self.holomap
@@ -243,12 +246,14 @@ class Renderer(Exporter):
         return data
 
 
-    def html(self, obj, fmt=None, css=None):
+    def html(self, obj, fmt=None, css=None, comm=True, **kwargs):
         """
         Renders plot or data structure and wraps the output in HTML.
+        The comm argument defines whether the HTML output includes
+        code to initialize a Comm, if the plot supplies one.
         """
         plot, fmt =  self._validate(obj, fmt)
-        figdata, _ = self(plot, fmt)
+        figdata, _ = self(plot, fmt, **kwargs)
         if css is None: css = self.css
 
         if fmt in ['html', 'json']:
@@ -268,7 +273,15 @@ class Renderer(Exporter):
         b64 = base64.b64encode(figdata).decode("utf-8")
         (mime_type, tag) = MIME_TYPES[fmt], HTML_TAGS[fmt]
         src = HTML_TAGS['base64'].format(mime_type=mime_type, b64=b64)
-        return tag.format(src=src, mime_type=mime_type, css=css)
+        html = tag.format(src=src, mime_type=mime_type, css=css)
+        if comm and plot.comm is not None:
+            comm, msg_handler = self.comms[self.mode]
+            msg_handler = msg_handler.format(comm_id=plot.comm.id)
+            return comm.template.format(init_frame=html,
+                                        msg_handler=msg_handler,
+                                        comm_id=plot.comm.id)
+        else:
+            return html
 
 
     def static_html(self, obj, fmt=None, template=None):
@@ -294,12 +307,9 @@ class Renderer(Exporter):
                 widget_type = 'scrubber'
             else:
                 widget_type = 'widgets'
-        elif dynamic == 'open': widget_type = 'scrubber'
-        elif dynamic == 'bounded': widget_type = 'widgets'
-        elif widget_type == 'widgets' and dynamic == 'open':
-            raise ValueError('Selection widgets not supported in dynamic open mode')
-        elif widget_type == 'scrubber' and dynamic == 'bounded':
-            raise ValueError('Scrubber widget not supported in dynamic bounded mode')
+        elif dynamic: widget_type = 'widgets'
+        elif widget_type == 'scrubber' and dynamic:
+            raise ValueError('DynamicMap do not support scrubber widget')
 
         if widget_type in [None, 'auto']:
             holomap_formats = self_or_cls.mode_formats['holomap'][self_or_cls.mode]
@@ -341,12 +351,13 @@ class Renderer(Exporter):
             widget = obj
 
         html = self_or_cls.static_html(widget, fmt, template)
+        encoded = self_or_cls.encode((html, {'mime_type': 'text/html'}))
         if isinstance(filename, BytesIO):
-            filename.write(html)
+            filename.write(encoded)
             filename.seek(0)
         else:
-            with open(filename, 'w') as f:
-                f.write(html)
+            with open(filename, 'wb') as f:
+                f.write(encoded)
 
 
     @classmethod
@@ -378,7 +389,7 @@ class Renderer(Exporter):
             backends = [cls.backend] if cls.backend else []
 
         # Get all the widgets and find the set of required js widget files
-        widgets = [wdgt for r in Renderer.__subclasses__()
+        widgets = [wdgt for r in [Renderer]+Renderer.__subclasses__()
                    for wdgt in r.widgets.values()]
         css = list({wdgt.css for wdgt in widgets})
         basejs = list({wdgt.basejs for wdgt in widgets})
@@ -386,10 +397,15 @@ class Renderer(Exporter):
 
         # Join all the js widget code into one string
         path = os.path.dirname(os.path.abspath(__file__))
-        widgetjs = '\n'.join(open(find_file(path, f), 'r').read()
-                             for f in basejs + extensionjs
-                             if f is not None )
-        widgetcss = '\n'.join(open(find_file(path, f), 'r').read()
+
+        def open_and_read(path, f):
+            with open(find_file(path, f), 'r') as f:
+                txt = f.read()
+            return txt
+
+        widgetjs = '\n'.join(open_and_read(path, f)
+                             for f in basejs + extensionjs if f is not None)
+        widgetcss = '\n'.join(open_and_read(path, f) 
                               for f in css if f is not None)
 
         dependencies = {}
@@ -457,11 +473,11 @@ class Renderer(Exporter):
             rendered = self_or_cls(plot, fmt)
         if rendered is None: return
         (data, info) = rendered
+        encoded = self_or_cls.encode(rendered)
         if isinstance(basename, BytesIO):
-            basename.write(data)
+            basename.write(encoded)
             basename.seek(0)
         else:
-            encoded = self_or_cls.encode(rendered)
             filename ='%s.%s' % (basename, info['file-ext'])
             with open(filename, 'wb') as f:
                 f.write(encoded)
@@ -502,3 +518,6 @@ class Renderer(Exporter):
         Loads any resources required for display of plots
         in the Jupyter notebook
         """
+        with param.logging_level('ERROR'):
+            cls.notebook_context = True
+
